/**
*
* Name		MNITEM -- Add, change, or remove an item in a menu.
*
* Synopsis	presult = mnitem (pmenu, row, col, option, pstring);
*
*		BMENU *presult	Pointer to menu which item and key(s)
*				were just added to, or NIL for failure.
*		BMENU *pmenu	Pointer to menu to which to add item.
*		int    row,	Row and column (relative to menu's
*		       col	window data area) at which to place
*				the item.
*		int    option	The bitwise OR-ing of the following values:
*				    MN_PROTECT
*					Item is protected.
*				    MN_NOPROTECT
*					Item is not protected.
*		char  *pstring	Text of the item.
*
* Description	This function writes an item to a menu.  An example
*		is probably best here:
*
*		The call:
*		    presult = mnitem (pmenu, 5, 2, MN_PROTECT, "Close");
*
*		Will add the text "Close" to the menu at (5,2).  The
*		item will be in the "protection" attribute, because
*		"option" is MN_PROTECT.
*
*		If row and col match an existing item, then MNITEM will
*		change the existing item if pstring is not NIL or the
*		NULL string (""), or it will remove the existing item
*		if pstring is NIL or the NULL string.
*
*		An error occurs if row or col exceeds the menu's
*		window dimensions.
*
* Returns	presult 		Pointer to changed menu, or
*					NIL if failure.
*		b_wnerr 	Possible values:
*				(No change)	Success.
*				MN_BAD_MENU	Pmenu is invalid.
*				MN_BAD_ITEM	A member of pmenu's item
*						list is bad.
*				WN_BAD_WIN	Pmenu's window is invalid.
*				WN_NO_MEMORY	Insufficient memory.
*				WN_ILL_DIM	Row or col out of range
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <string.h>

#include <bmenu.h>

#define TRUE  1
#define FALSE 0

#define NUL '\0'


BMENU *mnitem (pmenu, row, col, option, pstring)
BMENU *pmenu;
int    row, col, option;
char  *pstring;
{
    int      done = FALSE;
    BITEM   *pitem, *qitem;

		/* Validate menu data structures.		    */
    mnvalidm (pmenu)

		/* Make sure given row and col are within bounds.   */
    if (utrange (row, 0, pmenu->pwin->img.dim.h - 1) ||
	utrange (col, 0, pmenu->pwin->img.dim.w - 1))
	wnreterr (WN_ILL_DIM);

		/* Go through all existing menu items, and make     */
		/* qitem trail pitem through the list.		    */
    for (pitem = pmenu->pitems,  qitem = NIL;
	 (pitem != NIL) && (!done);
	 qitem = pitem,  pitem = pitem->next)
    {
		/* Check item signature byte.			    */
	if (pitem->signature != MN_ITEM_SIGN)
	    wnreterr (MN_BAD_ITEM);

		/* If existing row and column match, replace	    */
		/* existing item.				    */
	if ((pitem->row == row) && (pitem->col == col))
	{
		/* Erase the area used by the old item in the menu. */
	    wnscrblk (pmenu->pwin,
		      row, col, row, (col + pitem->len) - 1,
		      -1, -1, WNSCR_LEFT, 0, WN_UPDATE);

		/* Delete existing item if replacement string is    */
		/* NIL or the NUL-string.			    */
	    if ((pstring == NIL) || (*pstring == NUL))
	    {
		done = TRUE;

		/* Make previous item point at next item.	    */
		if (qitem == NIL)
		    pmenu->pitems = pitem->next;
		else
		    qitem->next = pitem->next;

		/* Delete item signature.			    */
		pitem->signature = MN_DEAD_ITEM;

		/* Free memory used by old item, and delete any key */
		/* bindings to this row and column.		    */
		free (pitem);
		mnkey (pmenu, row, col, 0, 0, 0, MN_DELETE);
	    }
	    else
	    {	/* The replacement string is not NULL, so do a true */
		/* replacement. 				    */
		done = TRUE;

		/* Figure out length of new item within bounds of   */
		/* window it has to fit in (or be trucated by).     */
		pitem->len = mntrunc (pmenu, pstring, col);

		/* Figure out protection value, record attribute.   */
		pitem->option = option;
		pitem->attr   = pmenu->pwin->attr;

		/* Write the text of the new item to the menu's     */
		/* window.					    */
		pmenu->pwin->cur_loc.col = col;
		pmenu->pwin->cur_loc.row = row;

		wnwrstrn (pmenu->pwin, pstring, pitem->len,
			 (option & MN_PROTECT)
			     ? (utlonyb (pmenu->proattr))
			     : (-1),
			 (option & MN_PROTECT)
			     ? (uthinyb (pmenu->proattr))
			     : (-1),
			 WN_UPDATE | CHARS_ONLY | WN_NO_SCROLL);
	    }
	}
    }

		/* If there is a non-NUL string to place, and we    */
		/* have not already done anything with it, it must  */
		/* be a string to be added... so we will add it.    */

		/* qitem points to the last item in the list, or is */
		/* NIL if there is no item list yet.		    */
    if ((!done) && (pstring != NIL) && (*pstring != NUL))
    {
		/* Attempt to allocate space for the new item.	    */
	if ((pitem = utalloc (BITEM)) == NIL)
	    wnreterr (WN_NO_MEMORY);

		/* Figure out length of new item within bounds of   */
		/* window it has to fit in (or be trucated by).     */
	pitem->len	= mntrunc (pmenu, pstring, col);

		/* Store values for future reference.		    */
	pitem->attr	= pmenu->pwin->attr;
	pitem->row	= row;
	pitem->col	= col;
	pitem->option	= option;
	pitem->plstring = NIL;
	pitem->next	= NIL;

		/* Create item signature.			    */
	pitem->signature= MN_ITEM_SIGN;

		/* If there is no item list, add this as the first  */
		/* item, otherwise add this item to the tail of the */
		/* item list.					    */
	if (pmenu->pitems == NIL)
	    pmenu->pitems = pitem;
	else
	    qitem->next = pitem;

		/* Write the text of the new item to the menu's     */
		/* window.					    */
	pmenu->pwin->cur_loc.col = col;
	pmenu->pwin->cur_loc.row = row;

	wnwrstrn (pmenu->pwin, pstring, pitem->len,
		 (option & MN_PROTECT)
		     ? (utlonyb (pmenu->proattr))
		     : (-1),
		 (option & MN_PROTECT)
		     ? (uthinyb (pmenu->proattr))
		     : (-1),
		 WN_UPDATE | CHARS_ONLY | WN_NO_SCROLL);

	done = TRUE;
    }

    return (pmenu);
}
