/**
*
* Name		MNLREAD - Read from a Lotus-style menu using a highlight
*			  bar, given a starting row and column.
*
* Synopsis	ret = mnlread (pmenu, srow, scol, prow, pcol,
*			       pch, pscan, option);
*
*		int ret 	WN_NO_ERROR (0) if selection was
*				without error, otherwise ret is
*				the MN/WN error code.
*		BMENU *pmenu	Pointer to menu to read from.
*		int    srow,	Starting row and column of item to
*		       scol	highlight first.
*		int   *prow,	Pointers to returned row and column
*		      *pcol	of highlight bar when a transmit
*				key is pressed.
*		int   *pch,	Pointers to returned character code
*		      *pscan,	and scan code of key pressed that
*				terminated MNREAD.
*		int    option	A bitwise OR-ing of the following
*				values:
*				    MN_ALL_TRANSMIT
*					Transmit on any key pressed,
*					whether it is a transmit key
*					or not.
*				    MN_DESTROY
*					Both remove the menu from
*					the screen and destroy the
*					menu's data structures upon
*					termination of MNREAD.
*				    MN_HOLD_BEEP
*					Don't beep if the key pressed
*					is a transmit key, even if
*					the key's action specifies a
*					beep.
*				    MN_KEEP_DESCRIPTION
*					Leave the "Lotus" description
*					on the menu on termination.
*				    MN_KEEP_HIGHLIGHT
*					Leave the highlight bar on
*					the menu on termination.
*				    MN_REMOVE
*					Remove the menu from the
*					display screen when MNLREAD
*					terminates.
*				    MN_UNKNOWN_BEEP
*					Beep when the user types an
*					unknown key.
*				    MN_UNKNOWN_TRANSMIT
*					Transmit when user types an
*					unknown key.
*
* Description	MNLREAD accepts keystrokes from the user and moves
*		a highlight bar on the specified menu.
*
*		An error code is returned if:
*		    There are no selectable items on the menu.
*		    The menu is not displayed (via MNDSPLAY).
*		    WNREMOVE fails (option includes MN_REMOVE
*			or MN_DESTROY).
*		    MNDSTROY fails (option includes MN_DESTROY).
*		    The key pressed is unknown (and option includes
*			MN_UNKNOWN_TRANSMIT).
*
* Returns	ret	WN_NO_ERROR (if no error), or a MN/WN error
*			code if failure.
*		*prow	The row and column where the highlight bar was
*		*pcol	when a transmit key was pressed.
*		*pch	The character code and scan code of the
*		*pscan	last key pressed.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <bkeybrd.h>
#include <bmenu.h>

#define TRUE  1
#define FALSE 0


int    mnlread	(pmenu, srow, scol, prow, pcol, pch, pscan, option)
BMENU *pmenu;
int    srow, scol;
int   *prow, *pcol;
int   *pch,  *pscan;
int    option;
{
    int      aoption,		/* The modification of the "option" */
				/*   passed to us which we pass on  */
				/*   to MNREADX.		    */
	     code,		/* Returned error code from MNFINDSL*/
	     done = FALSE,	/* Becomes true when MNLREAD should */
				/*   exit at next opportunity.	    */
	     olderr,		/* Storage for b_wnerr while we go  */
				/*   do clean-up window operations. */
	     result,		/* Return code from MNREAD.	    */
	     shouldbeep,	/* Whether we should beep or not.   */
	     was_delayed;	/* Whether the window was delayed   */
				/* when MNLREAD was called.	    */
    BITEM   *pitem,		/* Pointer to current menu item.    */
	    *plastitem; 	/* Pointer to last menu item.	    */
    BKEYMAP *pkey;		/* Pointer to current key found.    */
    BWINDOW *savecur;		/* Pointer to the previously-current*/
				/*   window.			    */

		/* Validate menu data structures.		    */
    if (mnvalmnu (pmenu) == NIL)
	wnretern (MN_BAD_MENU);

		/* Exit if window is covered or frozen. 	    */
    if (pmenu->pwin->internals.frozen)
	wnretern (WN_COVERED);

		/* Disable keys which do not point either at items  */
		/* or at (-1,-1).				    */
    if (mndisabl (pmenu) == NIL)
	return (b_wnerr);

		/* Find the first selectable menu item. 	    */
    if ((pitem = mnfindsl (pmenu, NIL, srow, scol, &code)) == NIL)
    {
	if (code == WN_NO_ERROR)
	    wnretern (MN_NONE_SELECT);
	else
	    return (b_wnerr);
    }

		/* Point pkey at menu's keylist.                    */
    pkey = pmenu->pkeys;

		/* Complete pending writes, and allow immediate I/O.*/
    was_delayed = b_pcurwin->options.delayed;
    if (wnupdate (b_pcurwin) == NIL)
	return (b_wnerr);
    b_pcurwin->options.delayed = 0;

		/* Save the starting row and column coordinates.    */
    *prow = pitem->row;
    *pcol = pitem->col;

		/* Save the current window for later restore.	    */
    savecur = NIL;
    if (wnvalwin (b_pcurwin) != NIL)
	savecur = b_pcurwin;

		/* Select menu's window, and save pointer to old    */
		/* current window.				    */
    if (wnselect (pmenu->pwin) == NIL)
    {
	olderr = b_wnerr;

	if (savecur != NIL)
	    wnselect (savecur);

	return (olderr);
    }

		/* Disable MN_REMOVE and MN_DESTROY for when we     */
		/* call MNREADX.  Set MN_ALL_TRANSMIT so we get     */
		/* control back after each key, and set MN_KEEPHIL  */
		/* so that the highlight bar doesn't flicker.       */
    aoption = ((option & ~MN_REMOVE) |
	       MN_ALL_TRANSMIT | MN_KEEP_HIGHLIGHT | MN_HOLD_BEEP);

		/* No last item yet, shouldn't beep yet...          */
    plastitem =  NIL;
    shouldbeep = FALSE;

    do		/* Erase previous "Lotus" description if there was  */
    {		/* one and it was different from the current one    */
		/* (i.e. highlight bar moved), then produce new     */
		/* "Lotus" item description.                        */
	if (plastitem != pitem)
	{
	    if (plastitem != NIL)
		wnscrblk (pmenu->pwin,
			  plastitem->lrow, plastitem->lcol,
			  plastitem->lrow,
			  (plastitem->lcol + plastitem->llen) - 1,
			  utlonyb (pmenu->pwin->attr),
			  uthinyb (pmenu->pwin->attr),
			  WNSCR_LEFT, 0, WN_UPDATE);

	    if (pitem->plstring != NIL)
		wnwrbuf (pitem->lrow, pitem->lcol,
			 pitem->llen, pitem->plstring,
			 utlonyb (pmenu->longattr),
			 uthinyb (pmenu->longattr),
			 CUR_AFTER | CHARS_ONLY);

		/* Store pointer to item so we will know if the     */
		/* highlight bar has moved next time we want to     */
		/* write out the "Lotus" description text.          */
	    plastitem = pitem;
	}

		/* Beep if we were told to do so on the last	    */
		/* iteration through the loop.			    */
	if (shouldbeep)
	{
	    scttywrt ('\a', 0);
	    shouldbeep = FALSE;
	}

		/* Call MNREADX to do one keystroke's worth of work.*/
		/* If MNREADX returns an error code, we are done.   */
	result = mnreadx (pmenu, &pitem, &pkey, pch, pscan, aoption);

	if ((result != WN_NO_ERROR) &&
	    (result != MN_READ_AB) &&
	    (result != MN_UNKNOWN_AB))
	    return (result);

		/* Check whether we should always transmit.	    */
	if (option & MN_ALL_TRANSMIT)
	    done = TRUE;

		/* Check whether key hit was a transmit key.	    */
	if (pkey != NIL)
	    if (pkey->action & MN_TRANSMIT)
		done = TRUE;

		/* Check whether we should beep, because we	    */
		/* prevented MNREADX from beeping.		    */
	if (((pkey == NIL) && (option & MN_UNKNOWN_BEEP)) ||
	    ((pkey != NIL) && (pkey->action & MN_BEEP)))
	    shouldbeep = TRUE;

		/* If we are not done, remove the highlighting on   */
		/* the last item.				    */
	if ((pitem != plastitem) && (!done))
	    if (mnatr (pmenu, plastitem, plastitem->attr) == NIL)
		return (b_wnerr);

		/* If we are done, store the final highlight bar    */
		/* coordinates. 				    */
	if (done)
	{
	    *pcol = pitem->col;
	    *prow = pitem->row;
	}
    } while (!done);

		/* Remove highlight bar if not told to keep it.     */
    if ( !(option & MN_KEEP_HIGHLIGHT))
	if (mnatr (pmenu, pitem, pitem->attr) == NIL)
	    return (b_wnerr);

		/* Remove description if not told to keep it.	    */
		/* Always delete old description if it is not the   */
		/* same item as it was previously.		    */
    if (plastitem != NIL)
	wnscrblk (pmenu->pwin,
		  plastitem->lrow, plastitem->lcol,
		  plastitem->lrow,
		  (plastitem->lcol + plastitem->llen) - 1,
		  utlonyb (pmenu->pwin->attr),
		  uthinyb (pmenu->pwin->attr),
		  WNSCR_LEFT, 0, WN_NO_UPDATE);

    if ((pitem->plstring != NIL) && (option & MN_KEEP_DESCRIPTION))
	wnwrbuf (pitem->lrow, pitem->lcol,
		 pitem->llen, pitem->plstring,
		 utlonyb (pmenu->longattr),
		 uthinyb (pmenu->longattr),
		 CUR_AFTER | CHARS_ONLY);

    else if (pitem->plstring != NIL)
	wnscrblk (pmenu->pwin,
		  pitem->lrow, pitem->lcol,
		  pitem->lrow,
		  (pitem->lcol + pitem->llen) - 1,
		  utlonyb (pmenu->pwin->attr),
		  uthinyb (pmenu->pwin->attr),
		  WNSCR_LEFT, 0, WN_NO_UPDATE);

				/* Restore "delayed" state.         */
    b_pcurwin->options.delayed = was_delayed;

		/* Reflect changes in window.			    */
    if (wnupdate (pmenu->pwin) == NIL)
	return (b_wnerr);

		/* Check whether we should beep, because we	    */
		/* prevented MNREADX from beeping.		    */
    if (shouldbeep && ( !(option & MN_HOLD_BEEP)))
	scttywrt ('\a', 0);

		/* Remove menu's window from display if told to.    */
    if (option & MN_REMOVE)
    {
	if (wnremove (pmenu->pwin) == NIL)
	    return (b_wnerr);
	if (savecur != NIL)
	    wnselect (savecur);
	savecur = NIL;
    }

		/* Destroy menu's data structures if told to do so. */
    if ((option & MN_DESTROY) == MN_DESTROY)
    {
	if (pmenu->pwin != savecur)
	{
	    if (mndstroy (pmenu) != WN_NO_ERROR)
		return (b_wnerr);
	    if (savecur != NIL)
		wnselect (savecur);
	}
	else if (mndstroy (pmenu) != WN_NO_ERROR)
	    return (b_wnerr);
    }

    return (result);
}
