/**
*
* Name		MNREAD - Read from a menu using a highlight bar,
*			 given a starting row and column.
*
* Synopsis	ret = mnread (pmenu, srow, scol, prow, pcol,
*			      pch, pscan, option);
*
*		int ret 	WN_NO_ERROR (0) if selection was
*				without error, otherwise ret is
*				the MN/WN error code.
*		BMENU *pmenu	Pointer to menu to read from.
*		int    srow,	Starting row and column of item to
*		       scol	highlight first.
*		int   *prow,	Pointers to returned row and column
*		      *pcol	of highlight bar when a transmit
*				key is pressed.
*		int   *pch,	Pointers to returned character code
*		      *pscan,	and scan code of key pressed that
*				terminated MNREAD.
*		int    option	A bitwise OR-ing of the following
*				values:
*				    MN_ALL_TRANSMIT
*					Transmit on any key pressed,
*					whether it is a transmit key
*					or not.
*				    MN_DESTROY
*					Both remove the menu from
*					the screen and destroy the
*					menu's data structures upon
*					termination of MNREAD.
*				    MN_HOLD_BEEP
*					Don't beep if the key pressed
*					is a transmit key, even if
*					the key's action specifies a
*					beep.
*				    MN_KEEP_HIGHLIGHT
*					Leave the highlight bar on
*					the menu on termination.
*				    MN_REMOVE
*					Remove the menu from the
*					display screen when MNREAD
*					terminates.
*				    MN_UNKNOWN_BEEP
*					Beep when the user types an
*					unknown key.
*				    MN_UNKNOWN_TRANSMIT
*					Transmit when user types an
*					unknown key.
*
*
* Description	MNREAD accepts keystrokes from the user and moves a
*		highlight bar on the specified menu.
*
*		An error code is returned if:
*		    There are no selectable items on the menu.
*		    The menu is not displayed (via MNDSPLAY).
*		    WNREMOVE fails (option includes MN_REMOVE
*			or MN_DESTROY).
*		    MNDSTROY fails (option includes MN_DESTROY).
*		    The key pressed is unknown (and option includes
*			MN_UNKNOWN_TRANSMIT).
*
* Returns	ret	WN_NO_ERROR (if no error), or a MN/WN error
*			code if failure.
*		*prow	The row and column where the highlight bar was
*		*pcol	when a transmit key was pressed.
*		*pch	The character code and scan code of the
*		*pscan	last key pressed.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <bmenu.h>


int    mnread	(pmenu, srow, scol, prow, pcol, pch, pscan, option)
BMENU *pmenu;
int    srow, scol;
int   *prow, *pcol;
int   *pch,  *pscan;
int    option;
{
    BITEM   *pitem;
    BKEYMAP *pkey;
    int      code, result;
    int      was_delayed;

		/* Validate menu data structures.		    */
    if (mnvalmnu (pmenu) == NIL)
	wnretern (MN_BAD_MENU);

		/* Exit if window is covered or frozen. 	    */
    if (pmenu->pwin->internals.frozen)
	wnretern (WN_COVERED);

		/* Complete pending writes, and allow immediate I/O.*/
    was_delayed = b_pcurwin->options.delayed;
    if (wnupdate (b_pcurwin) == NIL)
	return (b_wnerr);
    b_pcurwin->options.delayed = 0;

		/* Disable keys with MN_SELECT as their movement    */
		/* which have a (row,col) which do not match any    */
		/* selectable items.				    */
    if (mndisabl (pmenu) == NIL)
	return (b_wnerr);

		/* First find a selectable item to put highlight    */
		/* bar on.					    */
    if ((pitem = mnfindsl (pmenu, NIL, srow, scol, &code)) == NIL)
    {
	if (code == WN_NO_ERROR)
	    wnretern (MN_NONE_SELECT);
	else
	    return (code);
    }

		/* Point at menu's key list.                        */
    pkey = pmenu->pkeys;

		/* Call MNREADX to do all of the work.		    */
    result = mnreadx (pmenu, &pitem, &pkey, pch, pscan, option);

		/* Restore "delayed" state.                         */
    b_pcurwin->options.delayed = was_delayed;

    if ((result != WN_NO_ERROR)   &&
	(result != MN_READ_AB)	  &&
	(result != MN_UNKNOWN_AB))
	return (result);

		/* Return the final row and column of the highlight */
		/* bar. 					    */
    *pcol = pitem->col;
    *prow = pitem->row;

		/* Now destroy the menu if requested.		    */
    if ((option & MN_DESTROY) == MN_DESTROY)
	if (mndstroy (pmenu) != WN_NO_ERROR)
	    return (b_wnerr);

    return (result);
}
