/**
*
* Name		MNREADX - Read from a menu using a highlight bar,
*			  given a starting row and column, and
*			  a starting item pointer and keymap
*			  pointer.
*
* Synopsis	ret = mnreadx (pmenu, ppitem, ppkey,
*			       pch, pscan, option);
*
*		int ret 	WN_NO_ERROR (0) if selection was
*				without error, otherwise ret is
*				the MN/WN error code.
*		BMENU *pmenu	Pointer to menu to read from.
*		BITEM **ppitem	Pointer to a pointer to the "current"
*				item.
*		BKEYMAP **ppkey Pointer to a pointer to the "current"
*				key.
*		int   *pch,	Pointers to returned character code
*		      *pscan,	and scan code of key pressed that
*				terminated MNREAD.
*		int    option	A bitwise OR-ing of the following
*				values:
*				    MN_ALL_TRANSMIT
*					Transmit on any key pressed,
*					whether it is a transmit key
*					or not.
*				    MN_HOLD_BEEP
*					Don't beep if the key pressed
*					is a transmit key, even if
*					the key's action specifies a
*					beep.
*				    MN_KEEP_HIGHLIGHT
*					Leave the highlight bar on
*					the menu on termination.
*				    MN_REMOVE
*					Remove the menu from the
*					display screen when MNREADX
*					terminates.
*				    MN_UNKNOWN_BEEP
*					Beep when the user types an
*					unknown key.
*				    MN_UNKNOWN_TRANSMIT
*					Transmit when user types an
*					unknown key.
*
* Description	MNREADX accepts keystrokes from the user and moves a
*		highlight bar on the specified menu.
*
*		An error code is returned if:
*		    The item passed is not selectable or is NIL.
*		    The menu is not displayed (via MNDSPLAY).
*		    WNREMOVE fails (option includes MN_REMOVE
*			or MN_DESTROY).
*		    MNDSTROY fails (option includes MN_DESTROY).
*		    The key pressed is unknown (and option includes
*			MN_UNKNOWN_TRANSMIT).
*
* Returns	ret	WN_NO_ERROR (if no error);
*			MN_READ_AB if the key pressed had MN_ABORT;
*			MN_UNKNOWN_AB if the key was unknown and
*			    option had MN_UNKNOWN_TRANSMIT;
*			or a MN/WN error code if failure.
*		ppitem	Points to the now-current item.
*		ppkey	Points to the now-current key.
*		pch	The character code and scan code of the
*		pscan	last key pressed.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <bkeybrd.h>
#include <bmenu.h>

#define TRUE  1
#define FALSE 0

		/* Height of menu's data area.                      */
#define H (pmenu->pwin->img.dim.h)

		/* Width of menu's data area.                       */
#define W (pmenu->pwin->img.dim.w)


		/* Macro to do a for loop which makes the variable  */
		/* qitem take on all of the pointer values from     */
		/* pitem to the end of the item list in the menu's  */
		/* item list.  ritem is initialized to be pitem.    */

#define FORALL()    for (qitem = pmenu->pitems,  ritem = pitem;      \
			 qitem != NIL;				     \
			 qitem = qitem->next)			     \
		    {						     \
			if (qitem->signature != MN_ITEM_SIGN)	     \
			    wnretern (MN_BAD_ITEM);

#define FOREND()    }

		/* Macro which maximizes a "rangefunc".  This is    */
		/* used by MN_UP, DOWN, RIGHT, and LEFT to determine*/
		/* the correct place to move the highlight bar.     */
		/*     Arguments:				    */
		/*	   matchrc -- Either "row" or "col"; the    */
		/*		      component of the item's       */
		/*		      position which is supposed to */
		/*		      match that of the other item. */
		/*	   diffrc --  Either "row" or "col"; the    */
		/*		      opposite of "matchrc".        */
		/*	   size --    Either height (if matchrc is  */
		/*		      "col") or width (if matchrc is*/
		/*		      "row) of menu's window data   */
		/*		      area.			    */
		/*	   rangefunc- Either ULRANGE or DRRANGE.    */

#define IFMATCH(matchrc, diffrc, size, rangefunc)		     \
			if (( !(qitem->option & MN_PROTECT)) &&      \
			    (qitem->matchrc == pitem->matchrc) &&    \
			    (rangefunc (pitem, qitem, diffrc, size) >\
			     rangefunc (pitem, ritem, diffrc, size)))



#define DRRANGE(pitem, qitem, rowcol, size)			     \
	       ((qitem->rowcol <= pitem->rowcol)		     \
		 ? (pitem->rowcol - qitem->rowcol)		     \
		 : ((size - qitem->rowcol) + pitem->rowcol))

#define ULRANGE(pitem, qitem, rowcol, size)			     \
	       ((qitem->rowcol >= pitem->rowcol)		     \
		 ? (qitem->rowcol - pitem->rowcol)		     \
		 : ((size - pitem->rowcol) + qitem->rowcol))

static int mnchkxit (BKEYMAP *);


int    mnreadx (pmenu, ppitem, ppkey, pch, pscan, option)
BMENU	 *pmenu;
BITEM	**ppitem;
BKEYMAP **ppkey;
int	 *pch,	*pscan;
int	  option;
{
    int      code, done, shouldbeep;
    int      lasterr = b_wnerr;
    BITEM   *pitem, *qitem, *ritem, *plastitem;
    BKEYMAP *pkey,  *qkey;
    BWINDOW *savecur;

		/* Set-up starting values.			    */
    done       =  FALSE;
    shouldbeep =  FALSE;
    plastitem  =  NIL;
    pitem      = *ppitem;
    pkey       = *ppkey;

		/* First make sure given item is not NIL, then	    */
		/* check signature.				    */
    if (pitem == NIL)
	wnretern (WN_ILL_VALUE);
    if (pitem->signature != MN_ITEM_SIGN)
	wnretern (MN_BAD_ITEM);

		/* First make sure given key is not NIL, then check */
		/* signature.					    */
    if (pkey == NIL)
	wnretern (WN_ILL_VALUE);
    if (pkey->signature != MN_KEY_SIGN)
	wnretern (MN_BAD_KEY);

		/* Check to make sure there is an exit key.	    */
    if (mnchkxit (pmenu->pkeys))
	return (b_wnerr);

		/* Save the current window for later restore.	    */
    savecur = NIL;
    if (wnvalwin (b_pcurwin) != NIL)
	savecur = b_pcurwin;

		/* Select menu's window.                            */
    if (wnselect (pmenu->pwin) == NIL)
    {
	lasterr = b_wnerr;

	if (savecur != NIL)
	    wnselect (savecur);

	wnretern (lasterr);
    }

    do
    {		/* Un-highlight previous item if there was one and  */
		/* it was different from the current one (i.e.	    */
		/* highlight bar moved), then produce new highlight */
		/* bar. 					    */
	if (plastitem != pitem)
	{
	    if (plastitem != NIL)
		if (mnatr (pmenu, plastitem, plastitem->attr) == NIL)
		    return (b_wnerr);

		/* Now we want to highlight the item.		    */
	    if (mnatr (pmenu, pitem, pmenu->hilattr) == NIL)
		return (b_wnerr);

		/* Store pointer to item so we will know if the     */
		/* highlight bar has moved next time we want to do  */
		/* highlighting.				    */
	    plastitem = pitem;
	}

	/* Do a beep if the last keycode told us to do so....	    */
	if (shouldbeep)
	{
	    scttywrt ('\a', 0);
	    shouldbeep = FALSE;
	}

	/* Get a key from the user (complete keycode).		    */
	*pch = kbgetkey (pscan);

	/* Match key to an action.				    */
	if ((qkey = mnmchkey (pmenu, pkey, *pch, *pscan, &code)) != NIL)
	{
	    pkey = qkey;

		/* Do the "movement" part of the action.            */
	    switch (pkey->action & 0x0f)
	    {
		/* Do nothing.					    */
		case MN_NOMOVE:
		    break;


		/* Move highlight bar up.  Wrap to bottom if at top.*/
		case MN_UP:
		    FORALL ()
			IFMATCH (col, row, H, ULRANGE)
			    ritem = qitem;
		    FOREND ()

		    pitem = ritem;
		    break;


		case MN_DOWN:
		/* Move highlight bar down.  Wrap to top if at	    */
		/* bottom.					    */
		    FORALL ()
			IFMATCH (col, row, H, DRRANGE)
			    ritem = qitem;
		    FOREND ()

		    pitem = ritem;
		    break;


		case MN_RIGHT:
		/* Move highlight bar right.  Wrap to left if at    */
		/* rightmost item.				    */
		    FORALL ()
			IFMATCH (row, col, W, DRRANGE)
			    ritem = qitem;
		    FOREND ()

		    pitem = ritem;
		    break;


		case MN_LEFT:
		/* Move highlight bar left.  Wrap to right if at    */
		/* leftmost item.				    */
		    FORALL ()
			IFMATCH (row, col, W, ULRANGE)
			    ritem = qitem;
		    FOREND ()

		    pitem = ritem;
		    break;


		case MN_NEXT:
		/* Move highlight bar to "logical-next" menu        */
		/* position.  This is the next item in the order    */
		/* that items were entered by MNITEM and/or	    */
		/* MNITMKEY.  Wrap to beginning if at end.	    */
		    pitem = mnmchitm (pmenu, pitem->next, -1, -1, 0, &code);
		    if (code != WN_NO_ERROR)
			return (b_wnerr);
		    break;


		case MN_PREVIOUS:
		/* Move highlight bar to "logical-previous" menu    */
		/* position.  This is the previous item in the	    */
		/* order that items were entered by MNITEM and/or   */
		/* MNITMKEY.  Wrap to end if at beginning.	    */
		    if (pitem ==
			mnmchitm (pmenu, pmenu->pitems, -1, -1, 0, &code))
		    {
			FORALL ()
			    if (! (qitem->option & MN_PROTECT))
				ritem = qitem;
			FOREND ()
		    }
		    else
		    {
			if (code != WN_NO_ERROR)
			    return (b_wnerr);

			for (ritem = pitem,  qitem = pmenu->pitems;
			     qitem != pitem;
			     qitem = qitem->next)

			    if ( !(qitem->option & MN_PROTECT))
				ritem = qitem;
		    }
		    pitem = ritem;
		    break;


		case MN_FIRST:
		/* Move highlight bar to first item entered by	    */
		/* MNITEM and/or MNITMKEY.			    */
		    pitem = mnmchitm (pmenu, pmenu->pitems, -1, -1, 0, &code);
		    if (code != WN_NO_ERROR)
			return (b_wnerr);
		    break;


		case MN_LAST:
		/* Move highlight bar to last item entered by	    */
		/* MNITEM and/or MNITMKEY.			    */
		    FORALL ()
			if ( !(qitem->option & MN_PROTECT))
			    ritem = qitem;
		    FOREND ()

		    pitem = ritem;
		    break;

		case MN_SELECT:
		/* Move highlight bar to specified row/column of a  */
		/* menu item.					    */
		    if ((qitem =
			 mnmchitm (pmenu, pitem, pkey->row,
				   pkey->col, 0, &code)) != NIL)
			pitem = qitem;

		    else if (code != WN_NO_ERROR)
			return (b_wnerr);

		    break;
	    }

		/* Set up to do a BEEP if bit is set for this key.  */
	    if (pkey->action & MN_BEEP)
		shouldbeep = TRUE;

		/* If we should transmit, record the position where */
		/* the highlight bar sits, and set DONE to exit the */
		/* loop.					    */
	    if ((pkey->action & MN_TRANSMIT) ||
		(pkey->action & MN_ABORT)    ||
		(option & MN_ALL_TRANSMIT))
		done  = TRUE;
	}

	else	/* Unknown (or disabled) key pressed.  Or error     */
		/* walking key list.				    */
	{
	    if (code == WN_NO_ERROR)
	    {
		/* Check whether we should beep when unknown key    */
		/* pressed.					    */
		if (option & MN_UNKNOWN_BEEP)
		    shouldbeep = TRUE;

		/* Check whether we should transmit when unknown    */
		/* key pressed. 				    */
		if (option & MN_UNKNOWN_TRANSMIT)
		     done = TRUE;
	    }

	    else
		return (b_wnerr);
	}
    } while (!done);

		/* Remove old highlight bar if not told to keep it. */
    if (plastitem == pitem)
    {
	if ( !(option & MN_KEEP_HIGHLIGHT))
	    if (mnatr (pmenu, plastitem, plastitem->attr) == NIL)
		return (b_wnerr);
    }
    else
    {
	if (mnatr (pmenu, plastitem, plastitem->attr) == NIL)
	    return (b_wnerr);

	if (option & MN_KEEP_HIGHLIGHT)
	    if (mnatr (pmenu, pitem, pmenu->hilattr) == NIL)
		return (b_wnerr);
    }

		/* Do a beep if the last key (or unmatched key)     */
		/* asked for it and we have not done it yet.	    */
    if (shouldbeep & ( !(option & MN_HOLD_BEEP)))
	scttywrt ('\a', 0);

		/* Remove menu's window from display if told to.    */
    if (option & MN_REMOVE)
    {
	if (wnremove (pmenu->pwin) == NIL)
	    return (b_wnerr);
	if (savecur != NIL)
	    wnselect (savecur);
	savecur = NIL;
    }

		/* Save current values of pitem and pkey so callers */
		/* can use them if they desire. 		    */
    *ppitem = pitem;
    *ppkey  = pkey;

    if (b_wnerr != lasterr)
	return (b_wnerr);

		/* If a known key was pressed, and it was an abort  */
		/* key, abort!					    */
    if (pkey != NIL)
	if (pkey->action & MN_ABORT)
	    wnretern (MN_READ_AB);

		/* If we are supposed to transmit when we receive   */
		/* an unknown key, and we did receive an unknown    */
		/* key, transmit.				    */
    else if ((pkey == NIL) && (option & MN_UNKNOWN_TRANSMIT))
	wnretern (MN_UNKNOWN_AB);

    return (WN_NO_ERROR);
}



/**
*
* Name		mnchkxit -- Check whether there is any key which can
*			    exit the menu.
*
* Synopsis	ret = mnchkxit (pkey);
*
*		int	 ret	Zero if there is an exit key (i.e. no
*				error;	MN_BAD_KEY if a bad keymap
*				entry encountered;  MN_NO_EXIT if
*				there is no exit key.
*		BKEYMAP *pkey	Pointer to the start of the menu's
*				key list.
*
* Description	MNCHKXIT checks a menu's key list to see if there is
*		any way for the menu to be exited.  If there is at
*		least one keymap entry for which the MN_DISABLE bit
*		is not set, and which has either of the MN_TRANSMIT
*		or MN_ABORT bits set, then the function exits with
*		zero (no error).  If not, then the function exits
*		with MN_NO_EXIT.
*
* Returns	ret		Zero if there is an exit key (i.e. no
*				error;	MN_BAD_KEY if a bad keymap
*				entry encountered;  MN_NO_EXIT if
*				there is no exit key.
*
*		b_wnerr 	Possible values:
*				  (No change)	Success.
*				  MN_NO_EXIT	No way to exit the
*						menu.
*				  MN_BAD_KEY	Bad keymap entry
*						encountered.
**/

static int mnchkxit (pkey)
BKEYMAP *pkey;
{
    for (;  pkey != NIL;  pkey = pkey->next)
    {
		/* Check key signature. 			    */
	if (pkey->signature != MN_KEY_SIGN)
	    wnretern (MN_BAD_KEY);

	if ( !(pkey->action & MN_DISABLE)      &&
	     !(pkey->action & MN_TEMP_DISABLE) &&
	    ((pkey->action & MN_TRANSMIT)    ||
	     (pkey->action & MN_ABORT)))
	    return (0);
    }

    wnretern (MN_NO_EXIT);
}
