/**
*
* Name		scequip -- Obtain video hardware environment & settings
*
* Synopsis	model = scequip();
*
*		char model	 IBM Model code:
*				  IBM_PC (0xff) if IBM PC
*				  IBM_XT (0xfe) if IBM PC-XT
*						or Portable PC
*				  IBM_JR (0xfd) if IBM PCjr
*				  IBM_AT (0xfc) if IBM PC-AT
*				  IBM_CV (0xf9) if IBM PC Convertible
*				  IBM_30 (0xfa) if IBM PS/2 model 30
*				  IBM_50 (0xfc) if IBM PS/2 models 50 or 60
*				  IBM_80 (0xf8) if IBM PS/2 model 80
*
* Description	This function obtains information about the video
*		adapters installed on this machine and their options.
*		All of the results are available in global variables
*		declared in BSCREENS.H, except for b_pcmodel, which is
*		declared in BUTIL.H.
*
*		This function should need to be called only once during
*		the execution of a program.  (If it is called again, it
*		will examine the b_know_hw flag and skip most of its
*		operations to save time.)
*
*		Use SCMODE to obtain video information that might be
*		altered under software control (such as video mode or
*		number of columns).  Use SCNEWDEV to switch to a given
*		video adapter.
*
* Results	model	    IBM Model code:
*		b_pcmodel	  IBM_PC (0xff) if IBM PC
*				  IBM_XT (0xfe) if IBM PC-XT
*						or Portable PC
*				  IBM_JR (0xfd) if IBM PCjr
*				  IBM_AT (0xfc) if IBM PC-AT
*				  IBM_CV (0xf9) if IBM PC Convertible
*		(Also sets various global variables in BSCREENS.H)
*
*
* Version	5.00  (C)Copyright Blaise Computing Inc. 1986,1987
*
**/

#include <conio.h>
#include <dos.h>

#include <bscreens.h>

int b_know_hw = 0;	      /* Flag stating whether we have yet     */
			      /*     run SCEQUIP:  0 if no, 1 if yes  */

int b_mdpa = DONT_KNOW;       /* Monochrome Display & Printer Adapter */
			      /*   DONT_KNOW, ABSENT, or MONO	      */

int b_cga = DONT_KNOW;	      /* Color/Graphics Monitor Adapter       */
			      /*   DONT_KNOW, ABSENT, or COLOR	      */

int b_ega = DONT_KNOW;	      /* Enhanced Graphics Adapter	      */
			      /*   DONT_KNOW, ABSENT, MONO or COLOR   */

int b_pgc = DONT_KNOW;	      /* Professional Graphics Controller     */
			      /*   DONT_KNOW, ABSENT, COLOR or HIFUNC */

int b_mem_ega = 0;	      /* Amount of memory installed on EGA in */
			      /* 1024-byte units:  64, 128, 192, 256  */

unsigned b_sw_ega = 0x0ffff;  /* Switch settings on EGA 	      */
			      /* Low-order bit set means SW1 off, etc.*/
			      /* (Bit value 1 implies that	      */
			      /* corresponding switch is off.)	      */

static volatile int waiter;   /* Variable to assign to to wait for    */
			      /* CGA and PGC to stabilize when using  */
			      /* fast processors.		      */

#define  PGC_SEG   0xc600     /* Segment of memory-mapped PGC ports.  */
			      /* PGC port for presence test.	      */
#define  PGC_PRES_BYTE	(uttofar(PGC_SEG,0x03db,unsigned char))
#define  PGC_INDEX 0x03d4     /* Offset of PGC/CGA index port	      */
#define  PGC_IND_BYTE	(uttofar(PGC_SEG,PGC_INDEX,unsigned char))

#define WAIT							       \
    {								       \
	waiter	= 0;						       \
	waiter += 1;						       \
    }

static int adap_present(int);	      /* Internal function	      */
static int pgc_present(void);	      /* Internal function	      */
static int pgc_emulating(void);       /* Internal function	      */

char scequip()
{
    union REGS inregs,outregs;	      /* Registers for BIOS calls     */
    unsigned   color_or_mono,mem;
    int        pgc_mode;

    if (b_know_hw)
	return b_pcmodel;     /* No need to do this work again	      */

    if (utmodel() == IBM_JR)	      /* Fetch & save model code.     */
    {				      /* This is a PCjr 	      */
	b_mdpa = ABSENT;
	b_cga  = ABSENT;
	b_ega  = ABSENT;
	b_pgc  = ABSENT;
    }
    else if (b_pcmodel == IBM_CV)
    {				      /* IBM PC Convertible acts like */
	b_mdpa = MONO;		      /* both MDPA & CGA.	      */
	b_cga  = COLOR;
	b_ega  = ABSENT;
	b_pgc  = ABSENT;
    }
    else
    {				      /* Neither PCjr nor Convertible */

    /* First test for the presence of an EGA:  attempt BIOS video     */
    /* function 18, subfunction BL=0x10 ("Return EGA Information").   */
    /* If any information returned is invalid, then the EGA must be   */
    /* absent.							      */

	inregs.h.ah = 18;
	inregs.x.bx = 0xff10;
	inregs.x.cx = 0x000f;
	int86(16,&inregs,&outregs);
	color_or_mono = outregs.h.bh;
	mem	      = outregs.h.bl;
	b_sw_ega      = outregs.h.cl;

	if (b_sw_ega >= 12 || color_or_mono > 1 || mem > 3)
	    b_ega     = ABSENT;       /* Invalid value(s) found       */
	else
	{			      /* EGA is present 	      */
	    b_ega     = (color_or_mono ? MONO : COLOR);
	    b_mem_ega = 64 * (mem + 1);
	}

    /* If the EGA is present, the color_or_mono flag says whether     */
    /* it is emulating the MDPA or the CGA; if so, then the emulated  */
    /* board cannot itself be present, but the other might be.	      */

	if (b_ega == MONO
		|| !adap_present(0))	       /* Sense mono adapter  */
	    b_mdpa = ABSENT;
	else
	    b_mdpa = MONO;

    /* Further, if we sense a color adapter, then exactly one of the  */
    /* following conditions must be true:  EGA is in color mode       */
    /* (detected above from color_or_mono flag); PGC is emulating the */
    /* CGA; or a genuine CGA is present.			      */

	pgc_mode = DONT_KNOW;
	if (b_ega == COLOR
		|| (!adap_present(1))	       /* Sense color adapter */
		|| (pgc_mode = pgc_emulating()) == COLOR)
	    b_cga = ABSENT;
	else
	    b_cga = COLOR;

    /* If the PGC is present but not emulating the CGA, then it must  */
    /* be in its high-function graphics mode.			      */

	if (pgc_mode == COLOR)
	    b_pgc = COLOR;
	else
	    b_pgc = (pgc_present() ? HIFUNC : ABSENT);
    }

    b_know_hw = 1;
    return b_pcmodel;
}

/**
*
* Name		adap_present -- Test for Monochrome or Color/Graphics Adapter
*				or emulator
*
* Synopsis	present = adap_present(adapter);
*		int present	    1 if present or being emulated,
*				    0 if not.
*		int adapter	    0 if testing for Monochrome Adapter,
*				    1 if Color/Graphics Adapter
*
* Description	This function tests for the presence of the IBM
*		Monochrome Display and Printer Adapter or the IBM
*		Color/Graphics Adapter or other device (such as the EGA
*		or PGC) that may be emulating them.
*
*		We do an assignment between the UTINP and UTOUTP calls
*		so that the device has time to properly stabilize
*		when using slow devices with fast processors.
*
* Method	Write a bogus value into the cursor position register of
*		the CRT controller chip, then read it back again.  If
*		the same value is found in the register, then the CRTC
*		must be present; if the value is different, then it must
*		be absent.
*
* Result	present 	    1 if present or being emulated,
*				    0 if not.
*
**/

static int adap_present(adapter)
int adapter;
{
    int 	 crtc_there;
    char	 save_cursor;
    unsigned int port;	      /* Address of the CRT controller chip   */

    port = (adapter ? 0x03d4 : 0x03b4);

    utoutp(port++,0xf); 	      /* Set cursor register	      */
    WAIT
				      /* Save previous contents       */
    save_cursor = (unsigned char) utinp(port);

    utoutp(port,0x5a);		      /* Set to column 90	      */
    WAIT
    crtc_there = (utinp(port) == 0x5a);  /* Read the value back again */
    WAIT
    utoutp(port,save_cursor);	      /* Restore previous contents    */
    return crtc_there;
}

/**
*
* Name		pgc_present -- Test for IBM Professional Graphics Controller
*
* Synopsis	present = pgc_present();
*		int present	    1 if present, 0 if not.
*
* Description	This function tests for the presence of the IBM
*		Professional Graphics Controller.
*
* Method	Write a bogus value into the supposed PGC, then read it
*		back again.  If the same value is found, then the PGC
*		must be present; if the value is different, then it must
*		be absent.
*
* Result	present 	    1 if present, 0 if not.
*
**/

static int pgc_present()
{
    char save,dummy;

    save = utpeekb(PGC_PRES_BYTE);	/* Save previous contents     */
    WAIT
    utpokeb(PGC_PRES_BYTE,0x5a);	/* Store dummy value	      */
    WAIT
    dummy = utpeekb(PGC_PRES_BYTE);	/* Read it back again	      */
    WAIT
    utpokeb(PGC_PRES_BYTE,save);	/* Restore previous contents  */
    return (dummy == 0x5a);
}

/**
*
* Name		pgc_emulating -- Test for whether IBM Professional
*		    Graphics Controller is emulating the IBM Color/Graphics
*		    Adapter
*
* Synopsis	emulating = pgc_emulating();
*		int emulating	    COLOR  if PGC emulating the CGA,
*				    HIFUNC if not.
*
* Description	This function tests for whether the IBM Professional
*		Graphics Controller is emulating the IBM Color/Graphics
*		Adapter.
*
* Method	Write a bogus value into the PGC through a memory
*		address, then read it back again through an input port.
*		If the same value is found, then the PGC must be present
*		and in CGA emulation mode; if the value is different,
*		then (if the PGC is indeed present) it must be in
*		high-function graphics mode.
*
* Result	present 	    COLOR  if PGC emulating the CGA,
*				    HIFUNC if not.
*
**/

static int pgc_emulating()
{
    char save,dummy;

    save = utpeekb(PGC_IND_BYTE);	/* Save previous contents     */
    WAIT
    utpokeb(PGC_IND_BYTE,0x28); 	/* Store dummy value	      */
    WAIT
    dummy = (char) utinp(PGC_INDEX);	/* Read it back again	      */
    WAIT
    utpokeb(PGC_IND_BYTE,save); 	/* Restore previous contents  */

    return (dummy == 0x28) ? COLOR : HIFUNC;
}
