/**
*
* Name		WNATRBLK -- Change attributes on block of a window.
*
* Synopsis	presult = wnatrblk (pwin, r1, c1, r2, c2,
*				    fore, back, option);
*
*		BWINDOW *presult  Pointer to newly-changed BWINDOW
*				  structure, or NIL if failure.
*		BWINDOW *pwin	  Pointer to BWINDOW structure to
*				  change.
*		int r1, c1	  Row and column of upper-left corner
*				  of block to set attributes in
*				  (relative to upper-left corner of
*				  window).
*		int r2, c2	  Row and column of lower-right corner
*				  of block to set attributes in.
*		int fore	  New foreground attribute (-1 if
*				  foreground to be unchanged).
*		int back	  New background attribute (-1 if
*				  background to be unchanged).
*		int option	  WN_UPDATE to update the window (on the
*				  screen) after changing attributes;
*				  WN_NO_UPDATE to not update -- the
*				  attribute change will be in memory only.
*
* Description	This function fills a rectangular block of the data area
*		of a window with a given attribute without affecting
*		the characters already displayed there.  The cursor
*		is not moved.
*
*		If fore or back is -1, then foreground or background
*		attributes are left unchanged, respectively.
*
*		This function does not affect the window's default
*		attributes.  Use WNSETOPT for that purpose.
*
* Returns	presult 	  Pointer to newly-changed BWINDOW
*				  structure, or NIL if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   pwin is invalid.
*				  WN_NOT_SHOWN	   Internal error.
*				  WN_BAD_DEV	   Internal error.
*				  WN_ILL_DIM	   (R1,c1) or (r2,c2) not
*						   in window, or (r1,c1)
*						   not above-left of
*						   (r2,c2).
*				  WN_NULL_PTR	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <bwindow.h>

#define H      (pwin->img.dim.h)
#define W      (pwin->img.dim.w)
#define LENGTH ((c2 - c1) + 1)
#define START  ((W * r) + c1)


BWINDOW *wnatrblk (pwin, r1, c1, r2, c2, fore, back, option)
BWINDOW *pwin;
int	 r1, c1;
int	 r2, c2;
int	 fore, back;
int	 option;
{
    int   endpos, i, r;
    CELL *pdata;
    char  oldmask = '\0';
    char  attr;

		/* Validate window data structures.		    */
    wnvalidw (pwin)

		/* Make sure coordinates make sense.		    */
    if (utrange (r1, 0, H - 1) || utrange (r2, 0, H - 1) ||
	utrange (c1, 0, W - 1) || utrange (c2, 0, W - 1) ||
	(c1 > c2) || (r1 > r2))
	wnreterr (WN_ILL_DIM);

		/* "oldmask" will help us extract the portions of  */
		/* the existing attributes that we want to keep.   */
    if (fore == -1)
	oldmask  = 0x0f;
    if (back == -1)
	oldmask |= 0xf0;

    attr   = (char) (utnybbyt (back, fore) & ~oldmask);
    pdata  = pwin->img.pdata;

		/* Stuff the new attribute into the window.	    */
    for (r = r1;  r <= r2;  r ++)
    {
	endpos = (START + LENGTH);
	for (i = START;  i < endpos;  i ++)
	    pdata[i].attr = (pdata[i].attr & oldmask) | attr;
    }

		/* Write whole window unless delayed or not shown.  */
    if ( !(option & WN_NO_UPDATE))
	return (wnnupblk (pwin, r1, c1, r2, c2));

    pwin->internals.dirty = 1;
    return (pwin);
}
