/**
*
* Name		WNHIDE -- Conceal window but leave it attached to
*			  video display device and page.
*
* Synopsis	presult = wnhide (pwin);
*
*		BWINDOW *presult  Pointer to newly-hidden BWINDOW
*				  structure, or NIL if failure.
*		BWINDOW *pwin	  Pointer to BWINDOW structure to
*				  conceal.
*
* Description	This function makes a currently-displayed window
*		invisible and restores the previous contents of the
*		screen.  The window's location and border remain set so
*		the window may later be redisplayed via WNUNHIDE.
*
*		An error occurs if pwin does not point to a valid window
*		structure, or if the window has been designated "not
*		removable", or if the window is not currently displayed.
*
*		Use WNREMOVE to remove the window and detach it from its
*		location and border.  Use WNFORGET to detach it from its
*		location and border without restoring the screen.
*
* Returns	presult 	  Pointer to newly-hidden BWINDOW
*				  structure, or NIL if failure.
*		*pwin		  Several fields altered.
*		b_pactnode[][]	  Window node with active cursor.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   *pwin is invalid.
*				  WN_NOT_SHOWN	   Not currently shown.
*				  WN_CANT_HIDE	   Not removable.
*				  WN_BAD_NODE	   Internal error.
*				  WN_BAD_DEV	   Internal error.
*				  WN_ILL_DIM	   Internal error.
*				  WN_NULL_PTR	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>


/* Internal functions defined below:				      */

		/* Hide covering windows.			      */
static WIN_NODE *undisp(WIN_NODE *,LOC *,DIM *);

		/* Mark lower windows uncovered.		      */
static WIN_NODE *uncovr(WIN_NODE *,LOC *,DIM *);

		/* Check for covering windows.			      */
static int	 upcovr(WIN_NODE *,LOC *,DIM *);

		/* Redisplay covering windows.			      */
static WIN_NODE *redisp(WIN_NODE *);


BWINDOW *wnhide (pwin)
BWINDOW *pwin;
{
    int       old_dev, old_page, old_npage;
    int       mode, columns, act_page;
    BWINDOW  *presult;
    int       row, col, high, low;
    WIN_NODE *pnode;

    wnvalidw (pwin)

		/* Quit if already invisible.			    */
    if (pwin->options.hidden)
	return (pwin);

		/* Error if not removable.			    */
    if (!pwin->options.removable)
	wnreterr (WN_CANT_HIDE);

    old_page = b_curpage;
    old_dev  = scmode (&mode, &columns, &act_page);

		/* Validate and select device and page. 	    */
    if (wnseldev (&pwin->where_prev, &pwin->prev.dim, &old_npage))
	wnreterr (WN_NOT_SHOWN);

    presult = pwin;

		/* Temporarily remove higher windows covering this  */
		/* window.					    */
    if (undisp (pwin->pnode,
		&pwin->where_prev.corner,
		&pwin->prev.dim) == NIL)
	presult = NIL;

		/* Correct internal flag: this window was un-	    */
		/* displayed by UNDISP but NOT temporarily.	    */
    pwin->internals.temp_hid = 0;

    pwin->options.hidden = 1;

		/* Mark lower windows if uncovered.		    */
    if (uncovr (pwin->pnode,
		&pwin->where_prev.corner,
		&pwin->prev.dim) == NIL)
	presult = NIL;

		/* If this window has the active cursor...	    */
    if (b_pactnode[pwin->where_shown.dev][pwin->where_shown.page]
	->pwin == pwin)
    {
		/* Set so no window has an active cursor; retrieve  */
		/* cursor size; turn cursor off.		    */
	b_pactnode[pwin->where_shown.dev][pwin->where_shown.page] = NIL;
	sccurst (&row, &col, &high, &low);
	scpgcur (1, high, low, CUR_NO_ADJUST);
    }

    pwin->internals.cur_frozen = 1;

		/* Redisplay higher windows.  (also mark lower	    */
		/* windows if covered). 			    */
    if (redisp(pwin->pnode) == NIL)
	presult = NIL;

		/* Since we may have just exposed some windows,     */
		/* update each lower window on this display page,   */
		/* except those that are delayed.		    */
    for (pnode = b_wnlist[pwin->where_shown.dev][pwin->where_shown.page];
	 pnode != NIL;
	 pnode = pnode->below)

	if (!pnode->pwin->options.delayed)
	    if (wnupdate (pnode->pwin) == NIL)
	    {
		presult = NIL;
		break;
	    }

		/* Restore current page on new device; restore old  */
		/* device; restore current page on old device.	    */
    scpage (old_npage);
    scchgdev (old_dev);
    scpage (old_page);

    return (presult);
}


/**
*
* Name		undisp -- Temporarily remove ("un-display") windows
*			  covering a rectangular region on the screen.
*
* Synopsis	presult = *undisp (pnode, pcorner, pdim);
*
*		WIN_NODE *presult Copy of pnode, or NIL if failure.
*		WIN_NODE *pnode   Pointer to node of lowermost window
*				  to (possibly) remove.
*		LOC	 *pcorner Pointer to structure containing
*				  upper left corner of region.
*		DIM	 *pdim	  Pointer to structure containing
*				  dimensions of region.
*
* Description	This function finds all windows covering (directly or
*		indirectly) a rectangular region on a display screen and
*		attempts to temporarily remove them.  The windows
*		removed are flagged.
*
*		Hidden windows are not affected.
*
*		Non-removable windows are not actually removed.
*
* Returns	presult 	  Pointer to newly-hidden BWINDOW
*				  structure, or NIL if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_NODE	   Bad pnode or internal
*						   error.
*				  WN_BAD_DEV	   Internal error.
*				  WN_NULL_PTR	   Internal error.
*				  WN_ILL_DIM	   Internal error.
*
**/


static WIN_NODE *undisp (pnode, pcorner, pdim)
WIN_NODE	*pnode;
LOC		*pcorner;
DIM		*pdim;
{
    WIN_NODE *presult;
    BWINDOW  *pwin;

    wnvalidn (pnode)

    presult = pnode;
    pwin    = pnode->pwin;

		/* First un-display higher windows that cover this  */
		/* rectangle.					    */
    if (pnode->above != NIL)
	if (undisp (pnode->above,
		    pcorner,
		    pdim) == NIL)
	    presult = NIL;

		/* If this window covers the rectangle, undisplay.  */
    if ((!pwin->options.hidden)     &&
	(!pwin->internals.temp_hid) &&
	wnovrlap (pcorner, pdim,
		  &pwin->where_prev.corner, &pwin->prev.dim))
    {
		/* First un-display covering windows.		     */
	if (pnode->above != NIL)
	    if (undisp (pnode->above,
			&pwin->where_prev.corner,
			&pwin->prev.dim) == NIL)
		presult = NIL;

		/* Replace previous screen data.		    */
	if (pwin->options.removable)
	    if (wnputimg (&pwin->prev, &pwin->where_prev) == NIL)
		presult = NIL;

	pwin->internals.temp_hid = 1;	/* Temporarily hidden.	    */
	pwin->internals.dirty	 = 0;	/* No longer out-of-date.   */
    }

    return (presult);
}


/**
*
* Name		uncovr -- Mark windows uncovered which are below a given
*			  window and are overlapped by a given
*			  rectangular region but by no other windows.
*
* Synopsis	presult = *uncovr(pnode,pcorner,pdim);
*
*		WIN_NODE *presult Copy of pnode, or NIL if failure.
*		WIN_NODE *pnode   Pointer to node of uppermost window
*				  to (possibly) flag.
*		LOC	 *pcorner Pointer to structure containing
*				  upper left corner of region.
*		DIM	 *pdim	  Pointer to structure containing
*				  dimensions of region.
*
* Description	This function marks those windows as uncovered which are
*		at or below a given window and are overlapped by a given
*		rectangular region but by no other windows.
*
* Returns	presult 	  Pointer to newly-hidden BWINDOW
*				  structure, or NIL if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_NODE	   Bad pnode or internal
*						   error.
*
**/


static WIN_NODE *uncovr (pnode, pcorner, pdim)
WIN_NODE	*pnode;
LOC		*pcorner;
DIM		*pdim;
{
    WIN_NODE *presult;
    BWINDOW  *pwin;

    wnvalidn (pnode)
    pwin = pnode->pwin;

		/* If rectangle overlaps this window,		    */
    if (wnovrlap (pcorner,
		  pdim,
		  &pwin->where_shown.corner,
		  &pwin->img.dim))

		/* (unless higher windows overlap this one)	    */
	if (pnode->above == NIL ||
	    !upcovr (pnode->above,
		     &pwin->where_shown.corner,
		     &pwin->img.dim))

		/* mark this window "uncovered".                    */
	    pwin->internals.frozen	     =
	    pwin->internals.any_data_covered = 0;

    presult = pnode;
    if (pnode->below != NIL)	/* Continue with lower windows.     */
	if (uncovr (pnode->below, pcorner, pdim) == NIL)
	    presult = NIL;

    return (presult);
}


/**
*
* Name		upcovr -- Check whether higher windows overlap a
*			  rectangular region.
*
* Synopsis	result = *upcovr (pnode, pcorner, pdim);
*
*		WIN_NODE *presult Copy of pnode, or NIL if failure.
*		WIN_NODE *pnode   Pointer to node of window.
*		LOC	 *pcorner Pointer to structure containing
*				  upper left corner of region.
*		DIM	 *pdim	  Pointer to structure containing
*				  dimensions of region.
*
* Description	Given a window node and the rectangle defined by *pdim
*		and *pcorner, this function tells whether this window or
*		any higher window overlaps this rectangle.
*
* Returns	presult 	  Pointer to newly-hidden BWINDOW
*				  structure, or NIL if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_NODE	   Bad pnode or internal
*						   error.
*
**/

static int upcovr (pnode, pcorner, pdim)
WIN_NODE   *pnode;
LOC	   *pcorner;
DIM	   *pdim;
{
    int result;

    if (wnvalnod (pnode) == NIL)/* Quit if invalid node.	    */
	wnreterz (WN_BAD_NODE);

    if (!pnode->pwin->options.hidden &&
	wnovrlap (pcorner,	/* Check for rectangle overlap.     */
		  pdim,
		  &(pnode->pwin->where_prev.corner),
		  &(pnode->pwin->prev.dim)))
	result = 1;		/* Overlap.			    */

				/* No overlap, so look higher.	    */
    else if (pnode->above != NIL)
	result = upcovr (pnode->above, pcorner, pdim);

    else			/* No higher windows to check, so   */
	result = 0;		/* answer is "no".                  */

    return (result);
}


/**
*
* Name		redisp -- Redisplay windows at or above a given window
*			  window which have been temporarily hidden.
*
* Synopsis	presult = redisp (pnode, pcorner, pdim);
*
*		WIN_NODE *presult Copy of pnode, or NIL if failure.
*		WIN_NODE *pnode   Pointer to node of lowermost window
*				  to (possibly) redisplay.
*
* Description	This function redisplays windows at or above a given
*		window window which have been temporarily hidden.
*
*		Hidden windows are not affected.
*
*		Non-removable windows are redisplayed if they are
*		flagged as having been temporarily removed.
*
* Returns	presult 	  Pointer to newly-hidden BWINDOW
*				  structure, or NIL if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_NODE	   Bad pnode or internal
*						   error.
*				  WN_BAD_DEV	   Internal error.
*				  WN_NULL_PTR	   Internal error.
*				  WN_ILL_DIM	   Internal error.
*
**/


static WIN_NODE *redisp (pnode)
WIN_NODE	*pnode;
{
    BWINDOW *pwin;

    wnvalidn (pnode)
    pwin = pnode->pwin;

		/* Skip if hidden by user or if not temp. hidden.   */
    if ((!pwin->options.hidden) &&
	(pwin->internals.temp_hid))
    {
				/* Read previous screen contents.   */
	if (pwin->options.removable)
	    if (wngetimg (&pwin->prev,
			  &pwin->where_prev) == NIL)
		return (NIL);

				/* Write border.		    */
	wnputbor (&pwin->img.dim,
		  &pwin->bord,
		  &pwin->where_shown);

				/* Write data.			    */
	if (wnputimg (&pwin->img,
		      &pwin->where_shown) == NIL)
	    return (NIL);

	pwin->internals.dirty	 =	/* Output is now up-to-date.*/
	pwin->internals.temp_hid = 0;	/* No longer hidden.	    */

	if (pnode->below != NIL)/* Mark lower windows as covered.   */
	    if (wncover (pnode->below,
			 &pwin->where_shown.corner,
			 &pwin->prev.dim) == NIL)
		return (NIL);
    }

    if (pnode->above != NIL)	/* Continue with higher windows.    */
	if (redisp (pnode->above) == NIL)
	    return (NIL);

    return (pnode);
}
