/**
*
* Name		WNPIMBLK -- Write portion of window to screen.
*
* Synopsis	presult = wnpimblk (pimage, pwhere, r1, c1, r2, c2);
*
*		IMAGE *presult	Pointer to IMAGE structure written,
*				or NIL if failure.
*		IMAGE *pimage	Pointer to IMAGE structure to write
*				to screen.
*		WHERE *pwhere	Pointer to WHERE structure indicating
*				video device, display page, and
*				location on screen.
*		int r1, c1	Upper left coordinates (in window
*				data buffer) of portion of image
*				to update.
*		    r2, c2	Lower right coordinates (in window
*				data buffer) of portion of image
*				to update.
*
* Description	This function writes a portion of the contents of a
*		buffer specified by an IMAGE structure to a rectangular
*		region on a display screen.
*
*		The function also selects the designated device and
*		display page.
*
*		An error occurs if (1) the IMAGE structure pointed to by
*		pimage does not contain a valid pointer to a data
*		buffer, or if (2) the device or page designated by
*		pwhere are invalid, or if (3) the dimensions specified
*		by pimage->dim and the location specified by
*		pwhere->corner are inappropriate for the display device,
*		or if (4) the corner coordinates given are invalid.
*
* Returns	presult 	  Pointer to IMAGE structure read,
*				  or NIL if failure.
*		b_device	  Video device.
*		b_curpage	  Current display page.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_NULL_PTR	   pimage->pdata is
*						   invalid.
*				  WN_BAD_DEV	   Invalid device, page,
*						   or dimensions.
*				  WN_ILL_DIM	   Invalid coordinates or
*						   internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <stdio.h>

#include <bvideo.h>
#include <bwindow.h>

#define H (pimage->dim.h)
#define W (pimage->dim.w)


IMAGE *wnpimblk (pimage, pwhere, r1, c1, r2, c2)
IMAGE *pimage;
WHERE *pwhere;
int    r1, c1, r2, c2;
{
    int   old_npage, r;
    CELL *pfrom;

		/* If there is no IMAGE pointer, exit with error.   */
    if (pimage->pdata == NIL)
	wnreterr (WN_NULL_PTR);

		/* Validate and select device and and page (also    */
		/* validate dimensions. 			    */
    if (wnseldev (pwhere, &pimage->dim, &old_npage))
	wnreterr (WN_BAD_DEV);

		/* Make sure all coordinates are reasonable.	    */
    if (utrange (r1, 0, H - 1) || utrange (r2, 0, H - 1) ||
	utrange (c1, 0, W - 1) || utrange (c2, 0, W - 1) ||
	(c1 > c2) || (r1 > r2))
	wnreterr (WN_ILL_DIM);


		/* Write out each line-block of information.	    */
    pfrom = pimage->pdata + ((r1 * W) + c1);

    for (r = r1;
	 r <= r2;
	 r++,  pfrom += W)

	if (viwrrect (pwhere->corner.row + r,
		      pwhere->corner.col + c1,
		      pwhere->corner.row + r,
		      pwhere->corner.col + c1 + (c2 - c1),
		      (char *) pfrom,
		      0, 0, CHAR_ATTR) != (c2 - c1) + 1)
	    wnreterr (WN_ILL_DIM);

    return (pimage);
}
