/**
*
* Name		WNPUTBOR -- Display a border surrounding a rectangular
*			    region.
*
* Synopsis	result = wnputbor (pdim, pborder, pwhere);
*
*		int    result	  0 if the border is actually written,
*				  1 if not.
*		DIM    *pdim	  Pointer to DIM structure denoting
*				  dimensions of rectangle to surround.
*		WHERE  *pwhere	  Pointer to WHERE structure denoting
*				  display device, page, and location of
*				  upper left corner of rectangular region.
*		BORDER *pbord	  Pointer to BORDER structure denoting
*				  type of border to put around rectangle.
*
* Description	This function tries to display a border around a
*		rectangular region on a given video device and display
*		page.
*
*		The device and display page are made current (selected
*		for I/O).
*
*		If the display device or page or the dimensions of the
*		rectangle are illegal, no border is actually displayed
*		and 1 is returned as the value of the function.
*
* Results	result		  0 if the border is actually written,
*				  1 if not.
*		b_device	  Current display device.
*		b_curpage	  Current display page.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <string.h>

#include <bvideo.h>
#include <bwindow.h>

		/* Height and width of window data area.	    */
#define H	     (pdim->h)
#define W	     (pdim->w)

		/* Absolute screen ordinates for one-above top and  */
		/* one-below bottom of window data area.	    */
#define TOP_ROW      (pwhere->corner.row - 1)
#define BOTTOM_ROW   (TOP_ROW + H + 1)

		/* Absolute screen ordinates for one-to-left   and  */
		/* one-to-right of window data area.		    */
#define LEFT_COLUMN  (pwhere->corner.col - 1)
#define RIGHT_COLUMN (LEFT_COLUMN + W + 1)

		/* Display a string on screen, using absolute	    */
		/* coordinates and length.			    */
#define dispstr(row, col, pbuf, len, attr)			     \
		   (viwrrect((row),(col),(row),(col)+len-1,	     \
			    (pbuf),				     \
			    utlonyb (attr), uthinyb (attr),	     \
			    CHARS_ONLY))

		/* Display a string on screen, using absolute	    */
		/* coordinates and length.			    */
#define dispchar(row, col, pchar, attr) 			     \
		   (viwrrect((row),(col),(row),(col),		     \
			    (pchar),				     \
			    utlonyb (attr), uthinyb (attr),	     \
			    CHARS_ONLY))

		/* This is TRUE if all of the bits set in B are set */
		/* in A.					    */
#define IFBIT(a,b) if ((a & b) == b)


int wnputbor (pdim, pborder, pwhere)
DIM    *pdim;
BORDER *pborder;
WHERE  *pwhere;
{
    int     old_npage;
    int     mode, columns, act_page;
    int     bordertype = pborder->type & BBRD_TYPE;
    int     titletype  = pborder->type & BBRD_TITLE;
    int     tlen;
    BJOINT *pjoint;

		/* Select device and page.			    */
    if (wnseldev (pwhere, pdim, &old_npage))
	return (1);

		/* Exit if writing the border is impossible due to  */
		/* placement considerations.			    */
    if (bordertype == 0 	|| pwhere->corner.row <= 0 ||
	pwhere->corner.col <= 0 || BOTTOM_ROW >= scrows()  ||
	(scmode (&mode, &columns, &act_page), (RIGHT_COLUMN >= columns)))
	return (1);		/* No border actually written.	    */

		/* Write the border's box.                          */
    if (scbox (pwhere->corner.row - 1,
	       pwhere->corner.col - 1,
	       BOTTOM_ROW,
	       RIGHT_COLUMN,
	       (bordertype == BBRD_CHAR) ? (-1) : (bordertype - 1),
	       pborder->ch,
	       pborder->attr))
	return (1);

    if ((bordertype != BBRD_CHAR) || (titletype != BBRD_TITLE))
    {		/* Write out joints if any.			    */
	IFBIT (pborder->type, BBRD_HASJOINTS)
	{
	    for (pjoint = pborder->pjoints;
		 pjoint != NIL;
		 pjoint = pjoint->next)

		dispchar (pjoint->row + TOP_ROW,
			  pjoint->col + LEFT_COLUMN,
			  &(pjoint->jtype), pborder->attr);
	}

		/* Write out Top Left Title, if specified.	    */
	IFBIT (titletype, BBRD_TLT)
	{
	    tlen = strlen (pborder->pttitle);
	    tlen = (tlen > W) ? W : tlen;
	    dispstr (TOP_ROW, LEFT_COLUMN + 1,
		     pborder->pttitle, tlen, pborder->ttattr);
	}

		/* Write out Top Centered Title, if specified.	    */
	IFBIT (titletype, BBRD_TCT)
	{
	    tlen = strlen (pborder->pttitle);
	    tlen = (tlen > W) ? W : tlen;
	    dispstr (TOP_ROW,
		     LEFT_COLUMN +
			 (((RIGHT_COLUMN - LEFT_COLUMN) >> 1) -
			 (tlen >> 1)),
		     pborder->pttitle, tlen, pborder->ttattr);
	}

		/* Write out Top Right Title, if specified.	    */
	IFBIT (titletype, BBRD_TRT)
	{
	    tlen = strlen (pborder->pttitle);
	    tlen = (tlen > W) ? W : tlen;
	    dispstr (TOP_ROW, RIGHT_COLUMN - tlen,
		     pborder->pttitle, tlen, pborder->ttattr);
	}

		/* Write out Bottom Left Title, if specified.	    */
	IFBIT (titletype, BBRD_BLT)
	{
	    tlen = strlen (pborder->pbtitle);
	    tlen = (tlen > W) ? W : tlen;
	    dispstr (BOTTOM_ROW, LEFT_COLUMN + 1,
		     pborder->pbtitle, tlen, pborder->btattr);
	}

		/* Write out Bottom Centered Title, if specified.   */
	IFBIT (titletype, BBRD_BCT)
	{
	    tlen = strlen (pborder->pbtitle);
	    tlen = (tlen > W) ? W : tlen;
	    dispstr (BOTTOM_ROW,
		     LEFT_COLUMN +
			 (((RIGHT_COLUMN - LEFT_COLUMN) >> 1) -
			 (tlen >> 1)),
		     pborder->pbtitle, tlen, pborder->btattr);
	}

		/* Write out Bottom Right Title, if specified.	    */
	IFBIT (titletype, BBRD_BRT)
	{
	    tlen = strlen (pborder->pbtitle);
	    tlen = (tlen > W) ? W : tlen;
	    dispstr (BOTTOM_ROW, RIGHT_COLUMN - tlen,
		     pborder->pbtitle, tlen, pborder->btattr);
	}
    }
    return (0);
}
