/**
*
* Name		WNRDBUF -- Read series of adjacent characters
*			   from the current window.
*
* Synopsis	num_read = wnrdbuf (row, col, num_spaces, buffer, option);
*
*		int  num_read	Number of character cells actually read.
*		int  row, col	Row and column (relative to current
*				window's data area) at which to begin
*				reading.
*		int  num_spaces Number of character cells to read.
*		char *buffer	Space in which to put the data.
*		int  option	The option value tells WNRDBUF how the
*				buffer is constructed and where to
*				position the cursor after the buffer has
*				been read.  Three different bits are
*				relevant:
*
*		      Value	    Meaning
*		      -----------   ------------------------------------
*		      CUR_BEG	    Leave cursor at (row,col).
*		      CUR_AFTER     Leave cursor after end of string.
*
*		      CHARS_ONLY    Buffer contains characters only.
*		      CHAR_ATTR     Buffer contains (char,attr) pairs.
*
*		      MOVE_CUR	    Leave cursor according to
*				      CUR_BEG/CUR_AFTER bit.
*		      NO_MOVE_CUR   Preserve cursor location.
*
* Description	This function reads adjacent characters from the current
*		window with or without their corresponding video
*		attributes.  It can leave the window's cursor at the
*		beginning or end of the space read.
*
*		The num_spaces argument specifies the number of
*		"character cells" to read, where "character cells" means
*		the number of physical spaces on the screen.  (If
*		CHAR_ATTR is specified, twice this number of bytes will
*		be returned in the buffer.)  If num_spaces exceeds the
*		number of characters to the end of the row, the read
*		will continue on the following row, and so on to the end
*		of the window.	The read will stop at the end of the
*		window, hence the number of cells actually read may be
*		less than num_spaces.  The actual number is returned as
*		the value of the function.
*
*		An error occurs if no window is designated as current or
*		if row or col exceeds the window's dimensions.
*
*		If the read includes the lower right corner of the
*		window, then the cursor is left at the lower right
*		corner if MOVE_CUR and CUR_AFTER are both specified.
*
*		Be aware that this function does NOT add a trailing NUL
*		('\0') at the end of the buffer, and that any 8-bit
*		character may be present in the buffer, depending on
*		what is in the window.
*
* Returns	num_read	Number of character cells read
*				(0 if error)
*		*buffer 	Characters read from the window
*				(perhaps with their attributes)
*		b_pcurwin->cur_loc   Possibly altered.
*		b_wnerr 	Possible values:
*				(No change)	 Success.
*				WN_BAD_WIN	 b_pcurwin is invalid.
*				WN_ILL_DIM	 row or col out of range
*				WN_NULL_PTR	 Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>

#define H (b_pcurwin->img.dim.h)
#define W (b_pcurwin->img.dim.w)
#define AREA (H * W)


int wnrdbuf (row, col, num_spaces, buffer, option)
int   row, col, num_spaces;
char *buffer;
int   option;
{
    int   want_attr, i, offset, last;
    CELL *pdata;

		/* Validate window structure.			    */
    if (wnvalwin(b_pcurwin) == NIL)
	wnreterz (WN_BAD_WIN);

		/* Validate row, col.				    */
    if (utrange (row, 0, H - 1) ||
	utrange (col, 0, W - 1))
	wnreterz (WN_ILL_DIM);

		/* Be sure there's valid data.                      */
    if (b_pcurwin->img.pdata == NIL)
	wnreterz (WN_NULL_PTR);

		/* Cell at which to start.			    */
    offset = (row * W) + col;

		/* Pointer to first cell.			    */
    pdata = b_pcurwin->img.pdata + offset;

		/* Don't go beyond end of window.                   */
    utuplim (num_spaces, (AREA - offset))

    want_attr = ((option & CHAR_ATTR) != 0);

    for (i = 0; i < num_spaces; i++)
    {
	*buffer++ = pdata[i].ch;
	if (want_attr)
	    *buffer++ = pdata[i].attr;
    }

    if (!(option & NO_MOVE_CUR))
    {
	if (option & CUR_AFTER)
	{	/* Leave cursor after last read.		    */
	    last = offset + num_spaces;

		/* Don't go beyond end of window.                   */
	    utuplim (last, AREA - 1)
	    wncurmov (last / W, last % W);
	}
	else
	    wncurmov (row, col);
    }

    return (num_spaces);
}
