/**
*
* Name		WNSCRBLK -- Scroll the memory copy of a window.
*
* Synopsis	presult = wnscrblk (pwin, r1, c1, r2, c2,
*				    fore, back, dir, count, option);
*
*		BWINDOW *presult  Pointer to newly-scrolled BWINDOW
*				  structure, or NIL if failure.
*		BWINDOW *pwin	  Pointer to the window to scroll in.
*		int r1, c1	  Upper-left corner of region to scroll
*				  (relative to upper-left of window).
*		int r2, c2	  Lower-right corner of region to scroll
*				  (relative to upper-left of window).
*		int fore	  Foreground attribute of new blank lines.
*				  (-1 specifies window's native foreground
*				  color).
*		int back	  Background attribute of new blank lines.
*				  (-1 specifies window's native background
*				  color).
*		int dir 	  Scrolling direction (WNSCR_UP,
*				  WNSCR_DOWN, WNSCR_LEFT, or WNSCR_RIGHT).
*		int count	  Number of rows (or columns) to scroll.
*				  A value of 0 specifies that all
*				  characters lines within the region should
*				  be cleared.
*		int option	  WN_UPDATE to update the window (on the
*				  screen) after the scroll; WN_NO_UPDATE
*				  to not update -- the scroll will be in
*				  memory only.
*
* Description	This function scrolls rows/columns of characters (with
*		their attributes) up or down, left or right within the
*		specified subsection of a window.
*		The vacant rows are filled with blanks and a specified
*		attribute.
*
* Returns	presult 	  Pointer to newly-scrolled BWINDOW
*				  structure, or NIL if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   b_pcurwin is invalid.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <bwindow.h>

#define CELLSIZE (sizeof (CELL))

#define H (pwin->img.dim.h)	/* Height and width of data area    */
#define W (pwin->img.dim.w)

#define HR ((r2 - r1) + 1)	/* Height and width of region.	    */
#define WR ((c2 - c1) + 1)


BWINDOW *wnscrblk (pwin, r1, c1, r2, c2, fore, back, dir, count, option)
BWINDOW *pwin;
int r1, c1, r2, c2, fore, back, dir, count, option;
{
    int   attr, r, i;
    int   rt = r1, rb = r2, cl = c1, cr = c2;
    unsigned int amt;
    CELL *pto, *pfrom;

    wnvalidw (pwin)

		/* Check corners for in window, upper-left is up    */
		/* and to left of lower-right, etc.		    */
    if ((c1 > c2) || (r1 > r2) ||
	utrange (r1, 0, H - 1) || utrange (r2, 0, H - 1) ||
	utrange (c1, 0, W - 1) || utrange (c2, 0, W - 1))
	wnreterr (WN_ILL_DIM);

    attr = wndoattr (pwin->attr, fore, back);

		/* If request would scroll the region completely,   */
		/* set up to clear region.			    */
    if (((dir == WNSCR_UP || dir == WNSCR_DOWN)    &&
	 utrange (count, 1, HR - 1))		     ||
	((dir == WNSCR_LEFT || dir == WNSCR_RIGHT) &&
	 utrange (count, 1, WR - 1)))
	count = 0;

    else
	switch (dir)
	{
	    case WNSCR_UP:
		amt   = ((c2 - c1) + 1) * CELLSIZE;
		pto   = pwin->img.pdata + (W * r1)	     + c1;
		pfrom = pwin->img.pdata + (W * (r1 + count)) + c1;

		for (r = r1;
		     r <= r2 - count;
		     r++,  pto += W,  pfrom += W)
		    utmovmem ((char far *) pfrom,
			      (char far *) pto,
			      amt);

		rt = (r2 - count) + 1;
		break;

	    case WNSCR_DOWN:
		amt   = ((c2 - c1) + 1) * CELLSIZE;
		pto   = pwin->img.pdata + (W * (r2 + count)) + c1;
		pfrom = pwin->img.pdata + (W * r2)	     + c1;

		for (r = r2;
		     r >= r1 + count;
		     r--,  pto -= W,  pfrom -= W)
		    utmovmem ((char far *) pfrom,
			      (char far *) pto,
			      amt);

		rb = (r1 + count) - 1;
		break;

	    case WNSCR_LEFT:
		amt   = (((c2 - c1) - count) + 1) * CELLSIZE;
		pto   = pwin->img.pdata + (W * r1) + c1;
		pfrom = pwin->img.pdata + (W * r1) + c1 + count;

		for (r = r1;
		     r <= r2;
		     r++,  pto += W,  pfrom += W)
		    utmovmem ((char far *) pfrom,
			      (char far *) pto,
			      amt);

		cl = (c2 - count) + 1;
		break;

	    case WNSCR_RIGHT:
		amt   = (((c2 - c1) - count) + 1) * CELLSIZE;
		pto   = pwin->img.pdata + (W * r1) + c1 + count;
		pfrom = pwin->img.pdata + (W * r1) + c1;

		for (r = r1;
		     r <= r2;
		     r++,  pto += W,  pfrom += W)
		    utmovmem ((char far *) pfrom,
			      (char far *) pto,
			      amt);

		cr = (c1 + count) - 1;
		break;
	}


		/* Clear the block left over (or the whole block    */
		/* requested, if count was zero).		    */
    pto = pwin->img.pdata + (W * rt) + cl;

    for (r = rt;
	 r <= rb;
	 r++,  pto += W)

	for (i = 0;  i <= (cr - cl) + 1;  i++)
	{
	    pto[i].ch	= ' ';
	    pto[i].attr = (char) attr;
	}


		/* Update real screen if possible and requested.    */
    if ( !(option & WN_NO_UPDATE))
	return (wnnupblk (pwin, rt, cl, rb, cr));

    pwin->internals.dirty = 1;
    return (pwin);
}
