/**
*
* Name		WNWRBUF -- Write buffer to the current window.
*
* Synopsis	num_writ = wnwrbuf (row, col, num_spaces, buffer,
*				    fore, back, option);
*
*		int  num_writ	 Number of character cells actually
*				 written
*		int  row, col	 Row and column (relative to window's
*				 data area) at which to begin writing
*		int  num_spaces  Number of character cells to write.
*				 If CHARS_ONLY is specified (see "option"
*				 below), num_spaces == 0 indicates that
*				 the buffer is terminated by a NUL ('\0')
*				 character.
*		char *buffer	 Data to write
*		int  fore	 -1 if window's native foreground
*				 attribute is to be used;
*				 if option is 0 or 1, a value between 0 and
*				 15 specifies a new foreground attribute.
*		int  back	 -1 if window's native background
*				 attribute is to be used;
*				 if option is 0 or 1, a value between 0 and
*				 15 specifies a new background attribute.
*		int  option	 The option value tells WNWRBUF how the
*				 buffer is constructed and where to
*				 position the cursor after the buffer has
*				 been written.	Three different bits are
*				 relevant:
*
*		      Value	    Meaning
*		      -----------   ------------------------------------
*		      CUR_BEG	    Leave cursor at (row,col).
*		      CUR_AFTER     Leave cursor after end of string.
*
*		      CHARS_ONLY    Buffer contains characters only.
*				    If num_spaces is zero, the string
*				    is terminated by a NUL ('\0')
*				    character.
*		      CHAR_ATTR     Buffer contains (char,attr) pairs.
*
*		      MOVE_CUR	    Leave cursor according to
*				      CUR_BEG/CUR_AFTER bit.
*		      NO_MOVE_CUR   Preserve cursor location.
*
* Description	This function writes characters to the current window
*		with or without their corresponding video attributes.
*		It can leave the cursor at the beginning or end of the
*		space written to.
*
*		This routine will not scroll the window, although it
*		wraps text from one row to the next.
*
*		If the num_spaces argument is nonzero, it specifies the
*		number of "character cells" to write, where "character
*		cells" means the number of physical spaces on the
*		screen.  (If CHAR_ATTR is specified, twice this number
*		of bytes will be used from the buffer.)  If num_spaces
*		exceeds the number of characters to the end of the row,
*		the write will continue on the following row, and so on
*		to the end of the window.  The write will stop at the
*		end of the window, hence the number of cells actually
*		written may be less than what the buffer contains.  The
*		actual number is returned as the value of the function.
*
*		If the write includes the lower right corner of the
*		window, then the cursor is left at the lower right
*		corner if MOVE_CUR and CUR_AFTER are both specified.
*
*		Be aware that this function recognizes NULs ('\0') as
*		the end of the buffer only if num_spaces is zero and
*		CHARS_ONLY is specified.
*
*		An error occurs if no window is designated as current or
*		if row or col exceeds the window's dimensions.
*
* Returns	num_writ	Number of character cells written
*		b_pcurwin->cur_loc   Possibly altered.
*		b_wnerr 	Possible values:
*				(No change)	 Success.
*				WN_BAD_WIN	 b_pcurwin is invalid.
*				WN_ILL_DIM	 row or col out of range
*				WN_NULL_PTR	 Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <string.h>

#include <bwindow.h>

#define H (b_pcurwin->img.dim.h)
#define W (b_pcurwin->img.dim.w)
#define AREA (H * W)


int wnwrbuf (row, col, num_spaces, buffer, fore, back, option)
int   row, col, num_spaces;
char *buffer;
int   fore, back, option;
{
    int   want_attr, i, offset, last, last_row;
    int   attr;
    CELL *pdata;

		/* Validate current window's data structure.        */
    if (wnvalwin (b_pcurwin) == NIL)
	wnreterz (WN_BAD_WIN);

		/* Validate row, column.			    */
    if (utrange (row, 0, H - 1) || utrange (col, 0, W - 1))
	wnreterz (WN_ILL_DIM);

		/* Be sure there's valid data.                      */
    if (b_pcurwin->img.pdata == NIL)
	wnreterz (WN_NULL_PTR);

    attr      = wndoattr (b_pcurwin->attr, fore, back);
    want_attr = ((option & CHAR_ATTR) != 0);

    if ((!want_attr) && num_spaces == 0)
	num_spaces = (int) strlen (buffer);

		/* Update memory copy of the window's data area.    */
    offset = (row * W) + col;

		/* Pointer to first cell.			    */
    pdata  = b_pcurwin->img.pdata + offset;

		/* Don't go beyond end of window.                   */
    utuplim (num_spaces, (AREA - offset))

		/* "last" is offset of last char to write.          */
    last = offset + num_spaces;
    utuplim (last, AREA - 1);

		/* Last row written to window.			    */
    last_row = last / W;

    for (i = 0; i < num_spaces; i++)
    {		/* Copy to data in memory.			    */
	pdata[i].ch   = *buffer++;
	pdata[i].attr = (want_attr ? *buffer++ : (char) attr);
    }

		/* Update the physical screen if possible.	    */

		/* Update only a part of a line if we can get away  */
		/* with it.					    */
    if (row == last_row)
	wnnupblk (b_pcurwin, row, col, row, col + i);

    else	/* Otherwise update all lines changed.		    */
	wnnupblk (b_pcurwin, row, 0, last_row, W - 1);

    if (!(option & NO_MOVE_CUR))
    {
	if (option & CUR_AFTER)
		/* Leave cursor after last char.		   */
	    wncurmov (last_row, last % W);

	else	/* Leave cursor at beginning of string. 	    */
	    wncurmov (row, col);
    }

    return (num_spaces);
}
