/**
*
* Name		WNWRSTRN -- Write string of given length to a window.
*
* Synopsis	wnwrstrn (pwin, pbuffer, buflen, fore, back, option);
*
*		BWINDOW *pwin	  Window to write buffer into.
*		char	*pbuffer  Data to write.
*		int	 buflen   Amount of data to write.  If it is zero,
*				  and option includes CHARS_ONLY, then
*				  WNWRSTRN assumes the buffer is NUL-
*				  terminated.
*		int	 fore	  -1 if window's native foreground
*				  attribute is to be used;
*				  if option is 0 or 1, a value between 0 and
*				  15 specifies a new foreground attribute.
*		int	 back	  -1 if window's native background
*				  attribute is to be used;
*				  if option is 0 or 1, a value between 0 and
*				  15 specifies a new background attribute.
*		int	 option   The bitwise OR-ing of the following
*				  values:
*					WN_UPDATE
*					    Update window when through.
*					WN_NO_UPDATE
*					    Do not update window when
*					    through.
*					WN_SCROLL
*					    Scroll the window when the end
*					    of the last line is reached.
*					WN_NO_SCROLL
*					    Don't scroll the window.
*					    characters written after the
*					    end of the window buffer are
*					    lost.
*					WN_LF_LF
*					    Treat CR as CR, LF as LF--
*					WN_LF_CRLF
*					    Treat CR as CR, LF as CR/LF--
*					CHARS_ONLY
*					    Expect buffer to contain
*					    only characters.
*					CHAR_ATTR
*					    Expect (char,attr) pairs
*					    rather than just characters.
*
* Description	This function writes characters from a string to the
*		current window in TTY-style.
*
*		This routine will scroll the window if necessary, and
*		will also wrap text from one row to the next.
*
* Returns	pwin->cur_loc	Possibly altered.
*		b_wnerr 	Possible values:
*				(No change)	 Success.
*				WN_BAD_WIN	 pwin is invalid.
*				WN_NULL_PTR	 Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <stdio.h>
#include <string.h>

#include <bscreens.h>
#include <bwindow.h>

#define H      (pwin->img.dim.h)
#define W      (pwin->img.dim.w)
#define AREA   (H * W)
#define OFFSET ((row * W) + col)

#define BEL '\a'
#define BS  '\b'
#define CR  '\r'
#define NL  '\n'
#define NUL '\0'

#define TRUE 1
#define FALSE 0


void wnwrstrn (pwin, pbuffer, buflen, fore, back, option)
BWINDOW *pwin;
char	*pbuffer;
int	 buflen;
int	 fore, back;
int	 option;
{
    char  c;
    int   attr;
    int   row, col, srow, scol;
    int   i;
    int   locked = FALSE;
    CELL *pdata;

		/* Validate window data structure.		    */
    if (wnvalwin (pwin) == NIL)
	wnreterv (WN_BAD_WIN)

		/* If dirty already, update it if we should.	    */
    if ((pwin->internals.dirty) && (option & WN_UPDATE))
	wnneedup (pwin);

		/* Be sure there's valid data.                      */
    if (pwin->img.pdata == NIL)
	wnreterv (WN_NULL_PTR)

		/* Figure out length of string if CHARS_ONLY and    */
		/* the length is not given to us.		    */
    if ((buflen == 0) && ((option & CHAR_ATTR) == 0))
	buflen = strlen (pbuffer);

    attr = wndoattr (pwin->attr, fore, back);
    row  = srow = pwin->cur_loc.row;
    col  = scol = pwin->cur_loc.col;

		/* Pointer to first data cell.			    */
    pdata = pwin->img.pdata;

    for (i = OFFSET;
	 (buflen != 0) && (! locked);
	 buflen--,  (option & CHAR_ATTR) ? pbuffer += 2 : pbuffer++)

    {		/* Copy to data in memory.			    */
	switch (c = *pbuffer)
	{
	    case BEL:
		scttywrt ('\a', 0);
		break;

	    case BS:
		if (col != 0)
		{
		    col--;
		    i--;
		}
		break;

	    case NL:
		row++;

		if ( !(option & WN_LF_LF))
		    col = 0;

		i = OFFSET;
		break;

	    case CR:
		col = 0;
		i = OFFSET;
		break;

	    default:
		pdata[i].ch   = c;
		pdata[i].attr = (option & CHAR_ATTR)
				? *(pbuffer + 1)
				: (char) attr;
		col++;
		i++;
	}

	while ((row >= H) || (col >= W))
	{
	    if (col >= W)
	    {
		col = 0;
		row++;
	    }

	    if (row >= H)
	    {
		if (! (option & WN_NO_SCROLL))
		{
		    wnscrblk (pwin, 0, 0, H - 1, W - 1,
			      utlonyb (attr), uthinyb (attr),
			      WNSCR_UP, 1, WN_NO_UPDATE);

		    if ( !(option & WN_LF_LF))
			col = 0;

		    srow = 0;
		    row--;
		}

		else
		    locked = TRUE;
	    }
	    i = OFFSET;
	}
    }

		/* Move window cursor to next character position.   */
    pwin->cur_loc.row = row;
    pwin->cur_loc.col = (locked) ? 0 : col;

		/* Update the physical screen if possible and	    */
		/* requested.					    */
    if (! (option & WN_NO_UPDATE))
    {
	if (srow == row)
	    wnnupblk (pwin, srow, scol, row, col);
	else
	    wnnupblk (pwin, srow, 0, row, W - 1);
    }

    else
	pwin->internals.dirty = 1;
}
