#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/malloc.h>
#include <linux/sys.h>
#include <linux/fs.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <asm/io.h>

#define DEBUG
#define NODEBUG2

#define ADI_MAJOR_NUMBER 30	

char kernel_version[] = { "2.0.24" };

enum { /* IOCTL functions */
    ADI_INTERRUPT   = 1,
    ADI_SOFT_RESET  = 2,
    ADI_HARD_RESET  = 3,
    ADI_WAIT_IRQ    = 4
    };

/* write (adi_control) handshake bits */
#define F_ADS_BRK         0x0001  /* ads break */
#define F_HOST_REQ        0x0002  /* request flag */
#define F_HOST_ACK        0x0004  /* acknowledge flag */
#define F_ADS_ALL         0x0008  /* address all ads */
#define F_ADS_SELECT      0x0070  /* ADS select address */
#define F_ADS_RESET       0x0080  /* device reset */

/* write (adi_control/2) handshake bits */
#define F_INT_ACK         0x0040  /* interrupt acknowledege */

/* read (adi_status) hadshake bits */
#define F_ADS_INT         0x0001  /* ads service request */
#define F_ADS_REQ         0x0002  /* ads request */
#define F_ADS_ACK         0x0004  /* ads acknowledge */
#define F_HOST_BRK        0x0008  /* host service request */

struct ads {
    int handshake;  /* handshake port of ADI */
    int board;      /* ADS number */
    int data;       /* data port of ADI */
    int handshake2; /* handshake port/2 of ADI */
    int control;    /* ADI control bits */
    int control2;   /* ADI additional control bits */
    int status;     /* ADI status bits */
    };

enum { ADSCOUNT = 8 };

static struct ads ads_board [ ADSCOUNT ] =
{
    { 0x100, 0 },
    { 0x100, 1 },
    { 0x100, 2 },
    { 0x100, 3 },
    { 0x100, 4 },
    { 0x100, 5 },
    { 0x100, 6 },
    { 0x100, 7 }
    };

static inline void
adi_delay( short counter )
{
    while ( counter-- > 0 ) {
        asm( "nop" );
        asm( "nop" );
        }
    }

static inline void
init_board( struct ads* ads )
{
    /* zero handshake bits */

    outb( ads->control, ads->handshake );
    adi_delay( 10 );

    /* and set address select bits */
    ads->control = ( ads->board << 4 );
    }

static inline void 
write_data( struct ads* ads, int ch )
{
    outb( ch, ads->data );
    adi_delay( 10 );
    }

static inline int
read_data( struct ads* ads )
{
    return inb( ads->data );
    }

static inline void
select_board( struct ads* ads )
{
    outb( ads->control, ads->handshake );
    adi_delay( 10 );
    }

static inline void
assert_HOST_REQ( struct ads* ads )
{
    ads->control |= F_HOST_REQ;
    outb( ads->control, ads->handshake );
    adi_delay( 10 );
    }

static inline void
negate_HOST_REQ( struct ads* ads )
{
    ads->control &= ~F_HOST_REQ;
    outb( ads->control, ads->handshake );
    adi_delay( 10 );
    }

static inline void
assert_HOST_ACK( struct ads* ads )
{
    ads->control |= F_HOST_ACK;
    outb( ads->control, ads->handshake );
    adi_delay( 10 );
    }

static inline void
negate_HOST_ACK( struct ads* ads )
{
    ads->control &= ~F_HOST_ACK;
    outb( ads->control, ads->handshake );
    adi_delay( 10 );
    }

static inline void
assert_INT_ACK( struct ads* ads )
{
    ads->control2 |= F_INT_ACK;
    outb( ads->control2, ads->handshake2 );
    adi_delay( 10 );
    }

static inline void
negate_INT_ACK( struct ads* ads )
{
    ads->control2 &= ~F_INT_ACK;
    outb( ads->control2, ads->handshake2 );
    adi_delay( 10 );
    }

static inline void
assert_ADS_BRK( struct ads* ads )
{
    ads->control |= F_ADS_BRK;
    outb( ads->control, ads->handshake );
    adi_delay( 10 );
    }

static inline void
negate_ADS_BRK( struct ads* ads )
{
    ads->control &= ~F_ADS_BRK;
    outb( ads->control, ads->handshake );
    adi_delay( 10 );
    }

static inline void
assert_ADS_RESET( struct ads* ads )
{
    ads->control |= F_ADS_RESET;
    outb( ads->control, ads->handshake );
    adi_delay( 10 );
    }

static inline void
negate_ADS_RESET( struct ads* ads )
{
    ads->control &= ~F_ADS_RESET;
    outb( ads->control, ads->handshake );
    adi_delay( 10 );
    }

static inline int
ADS_ACK_asserted( struct ads* ads )
{
    ads->status = inb( ads->handshake );
    return ads->status & F_ADS_ACK;
    }

static inline int
ADS_REQ_asserted( struct ads* ads )
{
    ads->status = inb( ads->handshake );
    return ads->status & F_ADS_REQ;
    }

static inline int
ADS_INT_asserted( struct ads* ads )
{
    ads->status = inb( ads->handshake );
    return ads->status & F_ADS_INT;
    }

static inline int
HOST_BRK_asserted( struct ads* ads )
{
    ads->status = inb( ads->handshake );
    return ads->status & F_HOST_BRK;
    }

static int
adi_getc( struct ads* ads, int* ch )
{
    register int i;

    select_board( ads );
    i = 0;
    while ( ! ADS_REQ_asserted( ads ) ) {
        if ( i++ > 30000 ) {
            return 0;
            }
        }

    assert_HOST_ACK( ads );

    i = 0;
    while( ADS_REQ_asserted( ads ) ) {
        if( i++ > 30000 ) {
            printk( "ADI: Communication failure -- REQ stuck high\n" );
            negate_HOST_ACK( ads );
            return -1;
            }
        }

    *ch = read_data( ads );

    negate_HOST_ACK( ads );

    return 1;
    }

static int
adi_putc( struct ads* ads, register int ch )
{
    register int i;

    write_data( ads, ch );
    select_board( ads );
    assert_HOST_REQ( ads );

    i = 0;
    while( ! ADS_ACK_asserted( ads ) ) {
        if( i++ > 30000 ) {
            printk( "ADI: Communication failure -- ACK stuck low\n" );
            negate_HOST_REQ( ads );
            return 0;
            }
        }

    negate_HOST_REQ( ads );

    i = 0;
    while( ADS_ACK_asserted( ads ) ) {
        if( i++ > 30000 ){
            printk( "ADI: Communication failure -- ACK stuck high\n" );
            return 0;
            }
        }

    return 1;
    }

static void
adi_interrupt( struct ads* ads )
{
    select_board( ads );
    assert_ADS_BRK( ads );
    adi_delay( 100 );
    negate_ADS_BRK( ads ); 
    }

static void
adi_soft_reset( struct ads* ads )
{
    select_board( ads );
    assert_ADS_RESET( ads );
    /* delay at least 26 microseconds */
    {
        register int i = 0;
        for ( i = 0; i < 26; i ++ )
           schedule ();
        }
    negate_ADS_RESET( ads ); 
    }

static void
adi_hard_reset( struct ads* ads )
{
    select_board( ads );
    assert_INT_ACK( ads );
    assert_ADS_RESET( ads );
    /* delay at least 26 microseconds */
    {
        register int i = 0;
        for ( i = 0; i < 26; i ++ )
           schedule ();
        }
    negate_ADS_RESET( ads ); 
    negate_INT_ACK( ads );
    }

static int
adi_service_irq( struct ads* ads )
{
    register int svc = -1;
    register int i;

    if ( ! HOST_BRK_asserted( ads ) )
        return -1;

    select_board( ads );

    if ( ! ADS_INT_asserted( ads ) )
        return -1;

    assert_HOST_ACK( ads );
    svc = read_data( ads );
    negate_HOST_ACK( ads );
     
    assert_INT_ACK( ads ); 

    i = 0;
    while ( ! ADS_INT_asserted( ads ) ) {
        if ( i++ > 30000 ) {
            return -1;
            }
        }

    negate_INT_ACK( ads );

    return svc;
    }
    
/* ------------------- LINUX methods */

static int
adi_open( struct inode *inode, struct file *file )
{
    unsigned int minor = MINOR( inode->i_rdev );

    if ( minor >= ADSCOUNT ) {
#ifdef DEBUG
        printk( "ADI open: Minor number out of range!\n" );
#endif /* DEBUG */
        return -ENXIO;
        }

    {
        struct ads* ads = ads_board + minor;
#ifdef DEBUG
        printk( "ADI open: adi port 0x%03x, ads %d \n", ads->handshake, minor );
#endif /* DEBUG */

        init_board( ads_board + minor );
        }
        
    return 0;
    }


static void
adi_close( struct inode *inode, struct file *file )
{
    unsigned int minor = MINOR( inode->i_rdev );
    struct ads* ads = ads_board + minor;

#ifdef DEBUG
    printk( "ADI close: adi port 0x%03x, ads %d \n", ads->handshake, minor );
#endif /* DEBUG */
    }

static int
adi_ioctl( struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg )
{
    unsigned int minor = MINOR( inode->i_rdev );
    struct ads* ads = ads_board + minor;

#ifdef DEBUG
    printk( "ADI ioctl %d: adi port 0x%03x, ads %d \n", cmd, ads->handshake, minor );
#endif /* DEBUG */

    switch ( cmd ) {
    case ADI_INTERRUPT:
        adi_interrupt( ads ); 
        break;
    case ADI_SOFT_RESET:
        adi_soft_reset( ads ); 
        break;
    case ADI_HARD_RESET:
        adi_hard_reset( ads ); 
        break;
    case ADI_WAIT_IRQ:
        return adi_service_irq( ads );
    default:
        return -EINVAL;
        }

    return 0;
    }
 
static int
adi_write( struct inode *inode, struct file *file, const char *buf, int count )
{
    int xfercount = 0;
    unsigned int minor = MINOR( inode->i_rdev );
    struct ads* ads = ads_board + minor;

#ifdef DEBUG2
    printk( "ADI write: adi port 0x%03x, ads %d \n", ads->handshake, minor );
    printk( "adi_write: %d bytes\n", count );
#endif /* DEBUG2 */

    if ( verify_area( VERIFY_READ, buf, count ) )
        return -EINVAL;

    while ( count > 0 ) {
        int ch = get_fs_byte( buf );
        if ( ! adi_putc( ads, ch ) )
            return -EIO;
        count --;
        buf ++;
        xfercount ++;
        if ( ( xfercount % 64 ) == 0 )
            schedule ();
        }

    return xfercount;
    }

static int
adi_read( struct inode *inode, struct file *file, char *buf, int count )
{
    int xfercount = 0;
    unsigned int minor = MINOR( inode->i_rdev );
    struct ads* ads = ads_board + minor;

#ifdef DEBUG2
    printk( "adi_read: adi port 0x%03x, ads %d \n", ads->handshake, minor );
    printk( "adi_read: %d bytes\n", count );
#endif /* NODEBUG2 */

    if ( verify_area( VERIFY_WRITE, buf, count ) )
        return -EINVAL;

    while ( count > 0 ) {
        int ch = 0;
        int i = adi_getc( ads, &ch );
        if ( i < 0 )
            return -EIO;
        else if ( i == 0 )
            break;
        put_fs_byte( ch, buf );
        count --;
        buf ++;
        xfercount ++;
        if ( ( xfercount % 64 ) == 0 )
            schedule ();
        }

    return xfercount;
    }

static struct file_operations adi_fops =
{
    NULL,                                           /* lseek               */
    adi_read,                                       /* read                */
    adi_write,                                      /* write               */
    NULL,                                           /* readdir             */
    NULL,                                           /* select              */
    adi_ioctl,                                      /* ioctl               */
    NULL,                                           /* mmap                */
    adi_open,                                       /* open                */
    adi_close,                                      /* close               */
    NULL,                                           /* fsync               */
    NULL,                                           /* fasync              */
    NULL,                                           /* check_media_change  */
    NULL                                            /* revalidate          */
    };

/* ------------------- LINUX module initialization and cleanup */

int
init_module( void )
{
    register int i;

    printk( "ADI: %s %s %s\n", "[adi.c/v1.0]", __DATE__, __TIME__ );
    if ( register_chrdev( ADI_MAJOR_NUMBER, "adi", &adi_fops ) ) {
        printk( "Unable to get major number for ADI driver\n");
        return -EIO;
        }

    /* assume that handshake and board are configured in ads_board[] */
    for ( i = 0; i < ADSCOUNT; i ++ ) {
        struct ads* ads = ads_board + i;
        ads->data = ads->handshake + 1;
        ads->handshake2 = ads->handshake + 2;
        ads->control = 0;
        ads->control2 = 0;
        ads->status = 0;
        }

    printk( "ADI: Succesfully registered.\n" );
    return 0;
    }

void
cleanup_module( void )
{
    if ( unregister_chrdev( ADI_MAJOR_NUMBER, "adi" ) != 0 )
        printk( "ADI: Unregister failed\n" );
    else
        printk( "ADI: Unregister O.K.\n" );
    }

